<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Block\Adminhtml\Edit\Featuredcategories;
    use Magento\Framework\Data\Tree\Node;
    use Magento\Framework\App\ObjectManager;

    class Category extends \Magento\Catalog\Block\Adminhtml\Category\Tree  {

        protected $_selectedIds  = [];
        protected $_expandedPath = [];

        protected function _prepareLayout()     {
            $this->setTemplate("featuredcategories/categories.phtml");
        }

        public function getCategoryId()    {
            return ObjectManager::getInstance()->get("\Magento\Framework\Registry")->registry("categoryId") ? : null;
        }

        public function getCategoryIds()    {
            $categoryId = ObjectManager::getInstance()->get("\Magento\Framework\Registry")->registry("categoryId");
            $this->_selectedIds = [$categoryId];
            return $this->_selectedIds;
        }

        public function setCategoryIds($ids)    {
            if (empty($ids)) {
                $ids = [];
            } elseif (!is_array($ids)) {
                $ids = [(int)$ids];
            }
            $this->_selectedIds = $ids;
            return $this;
        }

        protected function getExpandedPath()    {
            return $this->_expandedPath;
        }

        protected function setExpandedPath($path)   {
            $this->_expandedPath = array_merge($this->_expandedPath, explode("/", $path));
            return $this;
        }

        protected function _getNodeJson($node, $level = 1)  {
            $item = [];
            $item["text"] = $this->escapeHtml($node->getName());
            if ($this->_withProductCount) {
                $item["text"] .= " (" . $node->getProductCount() . ")";
            }
            $item["id"] = $node->getId();
            $item["path"] = $node->getData("path");
            $item["cls"] = "folder " . ($node->getIsActive() ? "active-category" : "no-active-category");
            $item["allowDrop"] = false;
            $item["allowDrag"] = false;
            if (in_array($node->getId(), $this->getCategoryIds())) {
                $this->setExpandedPath($node->getData("path"));
                $item["checked"] = true;
            }
            if ($node->getLevel() < 2) {
                $this->setExpandedPath($node->getData("path"));
            }
            if ($node->hasChildren()) {
                $item["children"] = [];
                foreach ($node->getChildren() as $child) {
                    $item["children"][] = $this->_getNodeJson($child, $level + 1);
                }
            }
            if (empty($item["children"]) && (int)$node->getChildrenCount() > 0) {
                $item["children"] = [];
            }
            $item["expanded"] = in_array($node->getId(), $this->getExpandedPath());
            return $item;
        }

    }