<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Adminhtml\Notification;

    class Push extends \Webkul\Mobikul\Controller\Adminhtml\Notification  {

        public function execute()   {
            $resultRedirect  = $this->resultRedirectFactory->create();
            $notificationId  = $this->initCurrentNotification();
            if (!empty($notificationId)) {
                $model       = $this->_notificationRepository->getById($notificationId);
                $baseTmpPath = "mobikul/notification/";
                $target      = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA).$baseTmpPath;
                $bannerUrl   = "";
                if ($model->getFilename() != "")
                    $bannerUrl = $target.$model->getFilename();
                $message     = [
                    "id"               => $model->getId(),
                    "body"             => $this->getDecodedText($model->getContent()),
                    "title"            => $this->getDecodedText($model->getTitle()),
                    "sound"            => "default",
                    "message"          => $this->getDecodedText($model->getContent()),
                    "store_id"         => $model->getStoreId(),
                    "banner_url"       => $bannerUrl,
                    "notificationType" => $model->getType()
                ];
                if ($model->getType() == "category" && $model->getProCatId() != "") {
// for category /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                    $name = $this->_categoryResource->getAttributeRawValue($model->getProCatId(), "name", 1);
                    $message["categoryName"] = is_string($name) ? $name : "";
                    $message["categoryId"] = $model->getProCatId();
                } elseif ($model->getType() == "product" && $model->getProCatId() != "") {
// for product //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                    $name = $this->_productResource->getAttributeRawValue($model->getProCatId(), "name", 1);
                    $message["productName"] = is_string($name) ? $name : "";;
                    $message["productId"] = $model->getProCatId();
                }
                $url     = "https://fcm.googleapis.com/fcm/send";
                $authKey = $this->_helper->getConfigData("mobikul/notification/apikey");
                $headers = [
                    "Authorization : key=".$authKey,
                    "Content-Type  : application/json"
                ];
// for android //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                $topic  = $this->_helper->getConfigData("mobikul/notification/andoridtopic");
                $fields = [
                    "to"                => "/topics/".$topic,
                    "data"              => $message,
                    "priority"          => "high",
                    "time_to_live"      => 30,
                    "delay_while_idle"  => true,
                    "content_available" => true
                ];
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $this->_jsonHelper->jsonEncode($fields));
                $result = curl_exec($ch);
                curl_close($ch);
                if($this->isJson($result))
                    $this->messageManager->addSuccess(__("Notification pushed successfully for android."));
                else
                    $this->messageManager->addError(__("Sorry something went wrong for android."));
// for ios //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                $topic  = $this->_helper->getConfigData("mobikul/notification/iostopic");
                $fields = [
                    "to"                => "/topics/".$topic,
                    "data"              => $message,
                    "priority"          => "high",
                    "time_to_live"      => 30,
                    "notification"      => $message,
                    "delay_while_idle"  => true,
                    "content_available" => true,
                    "mutable_content"   => true
                ];
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $this->_jsonHelper->jsonEncode($fields));
                $result = curl_exec($ch);
                curl_close($ch);
                if($this->isJson($result))
                    $this->messageManager->addSuccess(__("Notification pushed successfully for iOS."));
                else
                    $this->messageManager->addError(__("Sorry something went wrong for iOS."));
            } else {
                $this->messageManager->addSuccess(__("Invalid Notification."));
            }
            return $resultRedirect->setPath("mobikul/notification/edit", ["id"=>$this->initCurrentNotification()]);
        }

        public function isJson($string) {
            json_decode($string);
            return (json_last_error() == JSON_ERROR_NONE);
        }

        protected function initCurrentNotification()    {
            return (int)$this->getRequest()->getParam("id");
        }

    }