<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Catalog;

    class CompareList extends AbstractCatalog  {

        public function execute()  {
            $returnArray                           = [];
            $returnArray["message"]                = "";
            $returnArray["otherError"]             = "";
            $returnArray["productList"]            = [];
            $returnArray["attributeValueList"]     = [];
            $returnArray["showSwatchOnCollection"] = false;
            try  {
                $wholeData      = $this->getRequest()->getPostValue();
                $this->_headers = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData)  {
                    $authKey  = $this->getRequest()->getHeader("authKey");
                    $authData = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1)  {
                        $width         = $wholeData["width"]         ?? 1000;
                        $storeId       = $wholeData["storeId"]       ?? 1;
                        $customerToken = $wholeData["customerToken"] ?? "";
                        $customerId    = $this->_helper->getCustomerByToken($customerToken) ?? 0;
                        $environment   = $this->_emulate->startEnvironmentEmulation($storeId);
// checking customer token ///////////////////////////////////////////////////////////////////////////////////////////////////////
                        if (!$customerId && $customerToken != "") {
                            $returnArray["message"]    = __("As customer you are requesting does not exist, so you need to logout.");
                            $returnArray["otherError"] = "customerNotExist";
                            $customerId = 0;           
                        } elseif ($customerId) {
                            $this->_customerSession->setCustomerId($customerId);
                        } 
// setting currency /////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $baseCurrency = $this->_store->getBaseCurrencyCode();
                        $currency     = $wholeData["currency"] ?? $baseCurrency;
                        $this->_store->setCurrentCurrencyCode($currency);
// checking is swatch allowed on colletion page /////////////////////////////////////////////////////////////////////////////////
                        $returnArray["showSwatchOnCollection"] = (bool)$this->_helper->getConfigData("catalog/frontend/show_swatches_in_product_list");
// getting compare list data ////////////////////////////////////////////////////////////////////////////////////////////////////
                        if ($this->_items === null)  {
                            $this->_compare->setAllowUsedFlat(false);
                            $this->_items = $this->_compareItemCollectionFactory->create();
                            $this->_items->useProductItem(true)->setStoreId($storeId);
                            if ($customerId != 0)
                                $this->_items->setCustomerId($customerId);
                            else
                                $this->_items->setVisitorId($this->_customerVisitor->getId());
                            $attributes = $this->_catalogConfig->getProductAttributes();
                            $this->_items
                                ->addAttributeToSelect($attributes)
                                ->loadComparableAttributes()
                                ->setVisibility($this->_productVisibility->getVisibleInSiteIds());
                        }
// getting product list /////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $productList = [];
                        foreach ($this->_items as $eachProduct)  {
                            $eachProduct = $this->_productFactory->create()->load($eachProduct->getId());
                            $productList[] = $this->_helperCatalog->getOneProductRelevantData($eachProduct, $storeId, $width, $customerId);
                        }
                        $returnArray["productList"] = $productList;
// getting attribute value list /////////////////////////////////////////////////////////////////////////////////////////////////
                        $block = $this->_compareListBlock;
                        $attributeValueList = [];
                        foreach ($this->_items->getComparableAttributes() as $attribute)  {
                            $eachRow = [];
                            $eachRow["attributeName"] = $this->_escaper->escapeHtml($attribute->getStoreLabel() ? $attribute->getStoreLabel() : __($attribute->getFrontendLabel()));
                            foreach ($this->_items as $item)  {
                                $eachItem = "";
                                switch ($attribute->getAttributeCode())  {
                                    case "price":
                                        $eachItem = $this->_helperCatalog->stripTags($this->_pricingHelper->currency($item->getFinalPrice()));
                                        break;
                                    case "small_image":
                                        $eachItem = $block->getImage($item, "product_small_image")->toHtml();
                                        break;
                                    default:
                                        $eachItem = $this->_catalogHelperOutput->productAttribute($item, $block->getProductAttributeValue($item, $attribute), $attribute->getAttributeCode());
                                        break;
                                }
                                $eachRow["value"][] = $eachItem;
                            }
                            $attributeValueList[] = $eachRow;
                        }
                        $returnArray["attributeValueList"] = $attributeValueList;
                        $this->_customerSession->setCustomerId(null);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else  {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else  {
                    $returnArray["message"] = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e)  {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }