<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Catalog;

    class ProductShare extends AbstractCatalog  {

        public function execute()  {
            $returnArray               = [];
            $returnArray["success"]    = false;
            $returnArray["message"]    = "";
            try  {
                $wholeData      = $this->getRequest()->getPostValue();
                $this->_headers = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData)  {
                    $authKey  = $this->getRequest()->getHeader("authKey");
                    $authData = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1)  {
                        $storeId        = $wholeData["storeId"]        ?? 1;
                        $message        = $wholeData["message"]        ?? "";
                        $productId      = $wholeData["productId"]      ?? 0;
                        $customerName   = $wholeData["customerName"]   ?? "";
                        $customerEmail  = $wholeData["customerEmail"]  ?? "";
                        $recipientName  = $wholeData["recipientName"]  ?? "[]";
                        $recipientEmail = $wholeData["recipientEmail"] ?? "[]";
                        $recipientName  = $this->_jsonHelper->jsonDecode($recipientName);
                        $recipientEmail = $this->_jsonHelper->jsonDecode($recipientEmail);
                        $environment    = $this->_emulate->startEnvironmentEmulation($storeId);
                        $senderData = [
                            "name"    => $customerName,
                            "email"   => $customerEmail,
                            "message" => $message
                        ];
                        $recipientData          = [];
                        $recipientData["name"]  = $recipientName;
                        $recipientData["email"] = $recipientEmail;
                        $product    = $this->_productFactory->create()->load($productId);
                        $sendFriend = $this->_sendFriend;
                        $sendFriend->setSender($senderData);
                        $sendFriend->setRecipients($recipientData);
                        $sendFriend->setProduct($product);
                        $validate = $sendFriend->validate();
                        if ($validate === true)  {
                            $sendFriend->send();
                            $returnArray["message"] = __("The link to a friend was sent.");
                            $returnArray["success"] = true;
                            $this->_emulate->stopEnvironmentEmulation($environment);
                            return $this->getJsonResponse($returnArray);
                        } else  {
                            if (is_array($validate))
                                $returnArray["message"] = implode(", ", $validate);
                            else
                                 $returnArray["message"] = __("We found some problems with the data.");
                        }
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else  {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else  {
                    $returnArray["message"] = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Magento\Framework\Exception\LocalizedException $e)  {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            } catch (\Exception $e)  {
                $returnArray["message"] = __("Some emails were not sent.");
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }