<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Catalog;

    use Magento\Catalog\Model\Layer\Resolver;

    class SearchResult extends AbstractCatalog  {

        public function execute()  {
            $returnArray                           = [];
            $returnArray["success"]                = false;
            $returnArray["message"]                = "";
            $returnArray["totalCount"]             = 0;
            $returnArray["otherError"]             = "";
            $returnArray["productList"]            = [];
            $returnArray["sortingData"]            = [];
            $returnArray["layeredData"]            = [];
            $returnArray["showSwatchOnCollection"] = false;
            try  {
                $wholeData      = $this->getRequest()->getPostValue();
                $this->_headers = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData)  {
                    $authKey  = $this->getRequest()->getHeader("authKey");
                    $authData = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1)  {
                        $width         = $wholeData["width"]         ?? 1000;
                        $storeId       = $wholeData["storeId"]       ?? 0;
                        $sortData      = $wholeData["sortData"]      ?? "[]";
                        $pageNumber    = $wholeData["pageNumber"]    ?? 1;
                        $filterData    = $wholeData["filterData"]    ?? "[]";
                        $searchQuery   = $wholeData["searchQuery"]   ?? "";
                        $customerToken = $wholeData["customerToken"] ?? "";
                        $customerId    = $this->_helper->getCustomerByToken($customerToken) ?? 0;
                        $filterData    = $this->_jsonHelper->jsonDecode($filterData);
                        $sortData      = $this->_jsonHelper->jsonDecode($sortData);
                        $environment   = $this->_emulate->startEnvironmentEmulation($storeId);
// checking customer token //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if (!$customerId && $customerToken != "")  {
                            $returnArray["message"]    = __("As customer you are requesting does not exist, so you need to logout.");
                            $returnArray["otherError"] = "customerNotExist";
                            $customerId = 0;
                        } elseif ($customerId) {
                            $this->_customerSession->setCustomerId($customerId);
                        }
// setting currency /////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $store         = $this->_store;
                        $currency      = $wholeData["currency"] ?? $store->getBaseCurrencyCode();
                        $store->setCurrentCurrencyCode($currency);
// checking is swatch allowed on colletion page /////////////////////////////////////////////////////////////////////////////////
                        $returnArray["showSwatchOnCollection"] = (bool)$this->_helper->getConfigData("catalog/frontend/show_swatches_in_product_list");
// searching product by query ///////////////////////////////////////////////////////////////////////////////////////////////////
                        $isFlatEnabled = $this->_productResourceCollection->isEnabledFlat();
                        $this->getRequest()->setParam("q", $searchQuery);

                        $query = $this->_queryFactory->get();
                        $query->setStoreId($storeId);
                        if ($query->getId())  {
                            $query->setPopularity($query->getPopularity()+1);
                        } else  {
                            $query->setQueryText($searchQuery)->setIsActive(1)->setIsProcessed(1)->setDisplayInTerms(1);
                            $query->setPopularity(1);
                        }
                        $query->prepare()->save();
                        $collection = $this->_helperCatalog->getProductListColl($this->_storeInterface->getStore()->getRootCategoryId(), 'search');
                        $layer = $this->_objectManager->create('\Magento\Catalog\Model\Layer\Search');
                        if ($this->_helperCatalog->showOutOfStock() == 0)
                            $this->_stockFilter->addInStockFilterToCollection($collection);
// Filtering product collection /////////////////////////////////////////////////////////////////////////////////////////////////
                        if (count($filterData) > 0)  {
                            for ($i=0; $i<count($filterData[0]); ++$i)  {
                                if ($filterData[0][$i] != "" && $filterData[1][$i] == "price")  {
                                    $priceRange      = explode("-", $filterData[0][$i]);
                                    $currencyRate    = $collection->getCurrencyRate();
                                    list($from, $to) = $priceRange;
                                    $collection->addFieldToFilter("price", ["from"=>$from, "to"=>empty($to) || $from == $to ? $to : $to - 0.001]);
                                    $layer->getState()->addFilter($this->_helperCatalog->_createItem(empty($from) ? 0 : $from, $to, $priceRange));
                                } elseif ($filterData[0][$i] != "" && $filterData[1][$i] == "cat")  {
                                    $categoryToFilter = $this->_category->create()->load($filterData[0][$i]);
                                    $collection->setStoreId($storeId)->addCategoryFilter($categoryToFilter);
                                } else  {
                                    $attribute = $this->_eavConfig->getAttribute("catalog_product", $filterData[1][$i]);
                                    $attributeModel = $this->_layerAttribute->create()->setAttributeModel($attribute);
                                    $collection->addFieldToFilter($attribute->getAttributeCode(), $filterData[0][$i]);
                                    $layer
                                    ->getState()
                                    ->addFilter($this->_helperCatalog->_createItem($filterData[0][$i], $filterData[0][$i]));
                                }
                            }
                        }
// Sorting product collection ///////////////////////////////////////////////////////////////////////////////////////////////////
                        if (count($sortData) > 0)  {
                            $sortBy = $sortData[0];
                            if ($sortData[1] == 0)
                                $collection->setOrder($sortBy, "ASC");
                            else
                                $collection->setOrder($sortBy, "DESC");
                        }
// Applying pagination //////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if ($pageNumber >= 1)  {
                            $returnArray["totalCount"] = $collection->getSize();
                            $pageSize = $this->_helperCatalog->getPageSize();
                            $collection->setPageSize($pageSize)->setCurPage($pageNumber);
                        }
// Creating product collection //////////////////////////////////////////////////////////////////////////////////////////////////
                        $productList = [];
                        foreach ($collection as $eachProduct)  {
                            $eachProduct = $this->_productFactory->create()->load($eachProduct->getId());
                            $productList[] = $this->_helperCatalog->getOneProductRelevantData($eachProduct, $storeId, $width, $customerId);
                        }
                        $returnArray["productList"] = $productList;
// Creating layered attribute collection ////////////////////////////////////////////////////////////////////////////////////////
                        $this->_mobikulLayer->_customCollection = $collection;
                        $this->_mobikulLayerPrice->_customCollection = $collection;
                        $doPrice = true;
                        if (count($filterData) > 0)  {
                            if (in_array("price", $filterData[1]))
                                $doPrice = false;
                        }
                        $layeredData = [];
                        $filters = $this->_filterableAttributes->getList();
                        foreach ($filters as $filter)  {
                            if ($filter->getFrontendInput() == "price")  {
                                if ($doPrice)  {
                                    $priceFilterModel = $this->_filterPriceDataprovider->create();
                                    if ($priceFilterModel)  {
                                        $each            = [];
                                        $each["code"]    = $filter->getAttributeCode();
                                        $each["label"]   = $filter->getStoreLabel();
                                        $each["options"] = $this->_helperCatalog->getPriceFilterOptions($filter, $collection);
                                        if (!empty($each["options"]))
                                            $layeredData[]   = $each;
                                    }
                                }
                            } else  {
                                $doAttribute = true;
                                if (count($filterData) > 0)  {
                                    if (in_array($filter->getAttributeCode(), $filterData[1]))
                                        $doAttribute = false;
                                }
                                if ($doAttribute)  {
                                    $attributeFilterModel = $this->_layerAttribute->create()->setAttributeModel($filter);
                                    if ($attributeFilterModel->getItemsCount())  {
                                        $each            = [];
                                        $each["code"]    = $filter->getAttributeCode();
                                        $each["label"]   = $filter->getStoreLabel();
                                        $each["options"] = $this->_helperCatalog->getFilterOptions($filter, $collection);
                                        if (!empty($each["options"]))
                                            $layeredData[]   =  $each;
                                    }
                                }
                            }
                        }
                        $returnArray["layeredData"] = $layeredData;
// Creating sort attribute collection ///////////////////////////////////////////////////////////////////////////////////////////
                        $sortingData       = [];
                        $toolbar           = $this->_toolbar;
                        foreach ($toolbar->getAvailableOrders() as $key=>$order)  {
                            $each          = [];
                            $each["code"]  = $key;
                            $each["label"] = __($order);
                            $sortingData[] = $each;
                        }
                        $returnArray["success"]     = true;
                        $returnArray["sortingData"] = $sortingData;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_customerSession->setCustomerId(null);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else  {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else  {
                    $returnArray["message"] = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e)  {
                $returnArray["message"] = __($e->getMessage());
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }