<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Checkout;

    class UpdateItemOptions extends AbstractCheckout    {

        public function execute()   {
            $returnArray               = [];
            $returnArray["message"]    = "";
            $returnArray["success"]    = false;
            $returnArray["cartCount"]  = 0;
            $returnArray["otherError"] = "";
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $qty             = $wholeData["qty"]             ?? 1;
                        $itemId          = $wholeData["itemId"]          ?? 0;
                        $params          = $wholeData["params"]          ?? "{}";
                        $quoteId         = $wholeData["quoteId"]         ?? 0;
                        $storeId         = $wholeData["storeId"]         ?? 1;
                        $productId       = $wholeData["productId"]       ?? 0;
                        $customerToken   = $wholeData["customerToken"]   ?? '';
                        $customerId      = $this->_helper->getCustomerByToken($customerToken) ?? 0;
                        $relatedProducts = $wholeData["relatedProducts"] ?? "[]";
                        $params          = $this->_jsonHelper->jsonDecode($params);
                        $relatedProducts = $this->_jsonHelper->jsonDecode($relatedProducts);
                        $environment     = $this->_emulate->startEnvironmentEmulation($storeId);
                        $quote           = new \Magento\Framework\DataObject();
// checking customer token //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if (!$customerId && $customerToken != "")  {
                            $returnArray["message"]    = __("As customer you are requesting does not exist, so you need to logout.");
                            $returnArray["otherError"] = "customerNotExist";
                            $customerId = 0;
                        }
// end checking customer token //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if ($customerId != 0) {
                            $quoteCollection = $this->_quoteFactory->create()->getCollection()
                                ->addFieldToFilter("customer_id", $customerId)
                                ->addFieldToFilter("is_active", 1)
                                ->addOrder("updated_at", "DESC");
                            $quote = $quoteCollection->getFirstItem();
                        }
                        if ($quoteId != 0)
                            $quote = $this->_quoteFactory->create()->setStoreId($storeId)->load($quoteId);
                        $quoteItem = $quote->getItemById($itemId);
                        if (!$quoteItem)
                            throw new \Magento\Framework\Exception\LocalizedException(__("We can't find the quote item."));
                        $product = $this->_productFactory->create()->load($productId);
                        $paramOption = [];
                        if (isset($params["options"])) {
                            $productOptions = $params["options"];
                            foreach ($productOptions as $optionId=>$values) {
                                $option     = $this->_productOption->load($optionId);
                                $optionType = $option->getType();
                                if (in_array($optionType, ["multiple", "checkbox"])) {
                                    foreach ($values as $optionValue)
                                        $paramOption[$optionId][] = $optionValue;
                                } elseif (in_array($optionType, ["radio", "drop_down", "area", "field"])) {
                                    $paramOption[$optionId] = $values;
                                } elseif ($optionType == "file") {
// downloading file /////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                    $base64String = $productOptions[$optionId]["encodeImage"];
                                    $fileName     = time().$productOptions[$optionId]["name"];
                                    $fileType     = $productOptions[$optionId]["type"];
                                    $fileWithPath = $this->_helperCatalog->getBasePath().DS.$fileName;
                                    $ifp          = fopen($fileWithPath, "wb");
                                    fwrite($ifp, base64_decode($base64String));
// assigning file to option /////////////////////////////////////////////////////////////////////////////////////////////////////
                                    $fileOption = [
                                        "type"       => $fileType,
                                        "title"      => $fileName,
                                        "quote_path" => DS."media".DS.$fileName,
                                        "fullpath"   => $fileWithPath,
                                        "secret_key" => substr(md5(file_get_contents($fileWithPath)), 0, 20)
                                    ];
                                    $filesToDelete[]        = $fileWithPath;
                                    $paramOption[$optionId] = $fileOption;
                                } elseif ($_optionType == "date") {
                                    $paramOption[$optionId]["month"]   = $values["month"];
                                    $paramOption[$optionId]["day"]     = $values["day"];
                                    $paramOption[$optionId]["year"]    = $values["year"];
                                } elseif ($_optionType == "date_time") {
                                    $paramOption[$optionId]["month"]   = $values["month"];
                                    $paramOption[$optionId]["day"]     = $values["day"];
                                    $paramOption[$optionId]["year"]    = $values["year"];
                                    $paramOption[$optionId]["hour"]    = $values["hour"];
                                    $paramOption[$optionId]["minute"]  = $values["minute"];
                                    $paramOption[$optionId]["dayPart"] = $values["dayPart"];
                                } elseif ($_optionType == "time") {
                                    $paramOption[$optionId]["hour"]    = $values["hour"];
                                    $paramOption[$optionId]["minute"]  = $values["minute"];
                                    $paramOption[$optionId]["dayPart"] = $values["dayPart"];
                                }
                            }
                        }
                        if(count($relatedProducts) == 0)
                            $relatedProducts = null;
                        if ($product->getTypeId() == "downloadable") {
                            if (isset($params))
                                $params = ["related_product"=>$relatedProducts, "links"=>$params["links"], "options"=>$paramOption, "qty"=>$qty, "product"=>$productId];
                            else
                                $params = ["related_product"=>$relatedProducts, "options"=>$paramOption, "qty"=>$qty, "product"=>$productId];
                        } elseif ($product->getTypeId() == "grouped") {
                            if(isset($params["super_group"]))
                                $params = ["related_product"=>$relatedProducts, "super_group"=>$params["super_group"], "product"=>$productId];
                        } elseif ($product->getTypeId() == "configurable") {
                            if(isset($params["super_attribute"]))
                                $params = ["related_product"=>$relatedProducts, "super_attribute"=>$params["super_attribute"], "options"=>$paramOption, "qty"=>$qty, "product_id"=>$productId];
                        } elseif ($product->getTypeId() == "bundle") {
                            if(isset($params["bundle_option"]) && isset($params["bundle_option_qty"])){
                                $this->_coreRegistry->register("product", $product);
                                $selectionCollection = $product->getTypeInstance(true)->getSelectionsCollection(
                                    $product->getTypeInstance(true)->getOptionsIds($product),
                                    $product
                                );
                                foreach ($selectionCollection as $option) {
                                    $selectionQty = $option->getSelectionQty() * 1;
                                    $key = $option->getOptionId();
                                    if (isset($params["bundle_option_qty"][$key]))
                                        $probablyRequestedQty = $params["bundle_option_qty"][$key];
                                    if ($selectionQty > 1)
                                        $requestedQty = $selectionQty * $qty;
                                    elseif (isset($probablyRequestedQty))
                                        $requestedQty = $probablyRequestedQty * $qty;
                                    else
                                        $requestedQty = 1;
                                    $associateBundleProduct = $this->_productFactory->create()->load($option->getProductId());
                                    $availableQty = $this->_stockRegistry->getStockItem($associateBundleProduct->getId())->getQty();
                                    if ($associateBundleProduct->getIsSalable()) {
                                        if ($requestedQty > $availableQty) {
                                            $returnArray["message"] = __("The requested quantity of ").$option->getName().__(" is not available");
                                            return $this->getJsonResponse($returnArray);
                                        }
                                    }
                                }
                                $params = ["related_product"=>$relatedProducts, "bundle_option"=>$params["bundle_option"], "bundle_option_qty"=>$params["bundle_option_qty"], "options"=>$paramOption, "qty"=>$qty, "product_id"=>$productId];
                            }
                        } else {
                            $params = ["related_product"=>$relatedProducts, "options"=>$paramOption, "qty"=>$qty, "product"=>$productId];
                        }
                        $item = $quote->updateItem($itemId, new \Magento\Framework\DataObject($params));
                        if (is_string($item))
                            throw new \Magento\Framework\Exception\LocalizedException(__($item));
                        if ($item->getHasError())
                            throw new \Magento\Framework\Exception\LocalizedException(__($item->getMessage()));
                        $item->setItemId($itemId)->save();
                        if (!$quote->getHasError()) {
                            $returnArray["success"] = true;
                            $returnArray["message"] = __(
                                "%1 was updated in your shopping cart.",
                                $this->_escaper->escapeHtml($item->getProduct()->getName())
                            );
                        }
                        $quote = $this->_quoteFactory->create()->setStoreId($storeId)->load($quoteId);
                        $returnArray["cartCount"] = $quote->getItemsQty() * 1;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $messages =  $e->getMessage();
                $returnArray["message"] = $messages;
                if (is_array($messages))
                    $returnArray["message"] = implode(", ", $messages);
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            } catch (\Exception $e) {
                $returnArray["message"] = __("We can't update the item right now.");
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }