<?php
    /*
    *
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class AddressBookData extends AbstractCustomer  {

        public function execute()   {
            $returnArray                             = [];
            $returnArray["message"]                  = "";
            $returnArray["otherError"]               = "";
            $returnArray["additionalAddress"]        = [];
            $returnArray["billingAddress"]["id"]     = 0;
            $returnArray["billingAddress"]["value"]  = __("You have no default billing address in your address book.");
            $returnArray["shippingAddress"]["id"]    = 0;
            $returnArray["shippingAddress"]["value"] = __("You have no default shipping address in your address book.");
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $storeId        = $wholeData["storeId"]       ?? 1;
                        $customerToken  = $wholeData["customerToken"] ?? "";
                        $customerId     = $this->_helper->getCustomerByToken($customerToken);
// checking customer token //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if (!$customerId && $customerToken != "")  {
                            $returnArray["otherError"] = "customerNotExist";
                            throw new \Magento\Framework\Exception\LocalizedException(
                                __("As customer you are requesting does not exist, so you need to logout.")
                            );
                        }
// end checking customer token ////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $customer    = $this->_customerFactory->create()->load($customerId);
                        $address     = $customer->getPrimaryBillingAddress();
                        if ($address) {
                            $returnArray["billingAddress"]["value"] = $address->format("html");
                            $returnArray["billingAddress"]["id"]    = $address->getId();
                        }
                        $address = $customer->getPrimaryShippingAddress();
                        if ($address) {
                            $returnArray["shippingAddress"]["value"] = $address->format("html");
                            $returnArray["shippingAddress"]["id"]    = $address->getId();
                        }
                        $additionalAddress = $customer->getAdditionalAddresses();
                        foreach ($additionalAddress as $eachAdditionalAddress) {
                            $eachAdditionalAddressArray = [];
                            if ($eachAdditionalAddress) {
                                $eachAdditionalAddressArray["id"]    = $eachAdditionalAddress->getId();
                                $eachAdditionalAddressArray["value"] = $eachAdditionalAddress->format("html");
                            }
                            else{
                                $eachAdditionalAddressArray["id"]    =  0;
                                $eachAdditionalAddressArray["value"] =  __("You have no other address entries in your address book.");
                            }
                            $returnArray["additionalAddress"][] = $eachAdditionalAddressArray;
                        }
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $returnArray["message"] = __($e->getMessage());
                return $this->getJsonResponse($returnArray);    
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }