<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class CreateAccount extends AbstractCustomer   {

        public function execute()   {
            $returnArray                         = [];
            $returnArray["success"]              = false;
            $returnArray["message"]              = "";
            $returnArray["cartCount"]            = 0;
            $returnArray["customerName"]         = "";
            $returnArray["customerEmail"]        = "";
            $returnArray["customerToken"]        = "";
            $returnArray["bannerDominantColor"]  = "";
            $returnArray["profileDominantColor"] = "";
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $os          = $wholeData["os"]         ?? "ios";
                        $dob         = $wholeData["dob"]        ?? "";
                        $email       = $wholeData["email"]      ?? "";
                        $width       = $wholeData["width"]      ?? 1000;
                        $token       = $wholeData["token"]      ?? "";
                        $mobile      = $wholeData["mobile"]     ?? "";
                        $prefix      = $wholeData["prefix"]     ?? "";
                        $suffix      = $wholeData["suffix"]     ?? "";
                        $taxvat      = $wholeData["taxvat"]     ?? "";
                        $gender      = $wholeData["gender"]     ?? "";
                        $mFactor     = $wholeData["mFactor"]    ?? 1;
                        $quoteId     = $wholeData["quoteId"]    ?? 0;
                        $storeId     = $wholeData["storeId"]    ?? 1;
                        $isSocial    = $wholeData["isSocial"]   ?? 0;
                        $password    = $wholeData["password"]   ?? "";
                        $lastName    = $wholeData["lastName"]   ?? "";
                        $websiteId   = $wholeData["websiteId"]  ?? 0;
                        $firstName   = $wholeData["firstName"]  ?? "";
                        $pictureURL  = $wholeData["pictureURL"] ?? "";
                        $middleName  = $wholeData["middleName"] ?? "";
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $emailValidator = new \Zend\Validator\EmailAddress();
                        if (!$emailValidator->isValid($email)) {
                            $returnArray["message"] = __("Invalid email address.");
                            return $this->getJsonResponse($returnArray);
                        }
                        $dob = $this->_localeDate->formatDate($dob);
                        $customerCheck = $this->_customerFactory->create()->setWebsiteId($websiteId)->loadByEmail($email);
                        $customerId = $customerCheck->getId();
                        if ($isSocial == 1 && $customerId > 0) {
                            $confirmationStatus = $this->_accountManagement->getConfirmationStatus($customerId);
                            if ($confirmationStatus === \Magento\Customer\Api\AccountManagementInterface::ACCOUNT_CONFIRMATION_REQUIRED) {
                                $returnArray["message"] = __("You must confirm your account. Please check your email for the confirmation link");
                                return $this->getJsonResponse($returnArray);
                            }
                            //updating customer password
                            $customer = $this->_customerRepositoryInterface->getById($customerId);
                            $customerSecure = $this->_customerRegistry->retrieveSecureData($customerId);
                            $customerSecure->setRpToken(null);
                            $customerSecure->setRpTokenCreatedAt(null);
                            $customerSecure->setPasswordHash($this->_encryptor->getHash($password, true));
                            $this->_customerRepositoryInterface->save($customer);

                            $returnArray["success"]       = true;
                            $returnArray["customerName"]  = $customerCheck->getName();
                            $returnArray["customerEmail"] = $customerCheck->getEmail();
                            $returnArray["message"]       = __("Your are now Loggedin");
                            $returnArray["customerToken"] = $this->_helper->getTokenByCustomerDetails($email, $password, $customerId);
                            $width *= $mFactor;
                            $height = ($width/2)*$mFactor;
                            $profileHeight = $profileWidth = 144 * $mFactor;
                            $collection = $this->_userImage->getCollection()->addFieldToFilter("customer_id", $customerId);
                            if ($collection->getSize() > 0) {
                                $time = time();
                                foreach ($collection as $value) {
                                    if ($value->getBanner() != "") {
                                        $basePath = $this->_baseDir.DS."mobikul".DS."customerpicture".DS.$customerId.DS.$value->getBanner();
                                        $newUrl = "";
                                        if(is_file($basePath)) {
                                            $newPath = $this->_baseDir.DS."mobikulresized".DS."customerpicture".DS.$customerId.DS.$width."x".$height.DS.$value->getBanner();
                                            $this->_helperCatalog->resizeNCache($basePath, $newPath, $width, $height);
                                            $newUrl = $this->_helper->getUrl("media")."mobikulresized".DS."customerpicture".DS.$customerId.DS.$width."x".$height.DS.$value->getBanner();
                                        }
                                        $returnArray["bannerImage"] = $newUrl."?".$time;
                                        $returnArray["bannerDominantColor"] = $this->_helper->getDominantColor($basePath);
                                    }
                                    if ($value->getProfile() != "") {
                                        $basePath = $this->_baseDir.DS."mobikul".DS."customerpicture".DS.$customerId.DS.$value->getProfile();
                                        $newUrl = "";
                                        if (is_file($basePath)) {
                                            $newPath = $this->_baseDir.DS."mobikulresized".DS."customerpicture".DS.$customerId.DS.$profileWidth."x".$profileHeight.DS.$value->getProfile();
                                            $this->_helperCatalog->resizeNCache($basePath, $newPath, $profileWidth, $profileHeight);
                                            $newUrl = $this->_helper->getUrl("media")."mobikulresized".DS."customerpicture".DS.$customerId.DS.$profileWidth."x".$profileHeight.DS.$value->getProfile();
                                        }
                                        $returnArray["profileImage"] = $newUrl."?".$time;
                                        $returnArray["profileDominantColor"] = $this->_helper->getDominantColor($basePath);
                                    }
                                }
                            }
                        } elseif ($customerId > 0) {
                            throw new \Magento\Framework\Exception\LocalizedException(
                                __("There is already an account with this email address.")
                            );
                        } else {
                            $customer = $this->_customerFactory->create();
                            $customerData = [
                                "dob"        => $dob,
                                "email"      => $email,
                                "prefix"     => $prefix,
                                "suffix"     => $suffix,
                                "taxvat"     => $taxvat,
                                "gender"     => $gender,
                                "lastname"   => $lastName,
                                "password"   => $password,
                                "website_id" => $websiteId,
                                "firstname"  => $firstName,
                                "middlename" => $middleName,
                                "group_id"   => $this->_helper->getConfigData(\Magento\Customer\Model\GroupManagement::XML_PATH_DEFAULT_ID)
                            ];
                            $this->getRequest()->setParams($customerData);
                            $customerObject = $this->_customerExtractor->extract("customer_account_create", $this->_request);
// Creating Customer ////////////////////////////////////////////////////////////////////////////////////////////////////////////
                            $customer      = $this->_accountManagement->createAccount($customerObject, $password, "");
                            $customerId    = $customer->getId();
                            $customer      = $this->_customerFactory->create()->load($customerId);
// Setting Social Data //////////////////////////////////////////////////////////////////////////////////////////////////////////
                            $returnArray["customerName"] = $customer->getName();
                            if ($isSocial == 1) {
                                $this->_userImage
                                    ->setProfile($pictureURL)
                                    ->setCustomerId($customerId)
                                    ->setIsSocial(1)
                                    ->save();
                                $returnArray["success"]       = true;
                                $returnArray["customerEmail"] = $email;
                                $returnArray["message"]       = __("Your Account has been successfully created");
                                $returnArray["customerToken"] = $this->_helper->getTokenByCustomerDetails($email, $password, $customerId);
                            }
                        }
                        $customer = $this->_customerRepositoryInterface->getById($customerId);
                        if ($quoteId != 0) {
                            $guestQuote = $this->_quoteFactory->create()->setStoreId($storeId)->load($quoteId);
                            $customerQuote = $this->_quote->getCollection()
                                ->addFieldToFilter("customer_id", $customerId)
                                ->addFieldToFilter("is_active", 1)
                                ->addOrder("updated_at", "DESC")
                                ->getFirstItem();
                            if ($customerQuote->getId() > 0) {
                                $customerQuote->merge($guestQuote)->collectTotals()->save();
                            } else {
                                $guestQuote->assignCustomer($customer)
                                    ->setCustomer($customer)
                                    ->getShippingAddress()
                                    ->setCollectShippingRates(true);
                                $guestQuote->collectTotals()->save();
                            }
                        }
                        $confirmationStatus = $this->_accountManagement->getConfirmationStatus($customer->getId());
                        if ($confirmationStatus === \Magento\Customer\Api\AccountManagementInterface::ACCOUNT_CONFIRMATION_REQUIRED) {
                            throw new \Magento\Framework\Exception\LocalizedException(
                                __("You must confirm your account. Please check your email for the confirmation link")
                            );
                        }
                        $quote = $this->_quote->getCollection()
                            ->addFieldToFilter("customer_id", $customerId)
                            ->addFieldToFilter("is_active", 1)
                            ->addOrder("updated_at", "DESC")
                            ->getFirstItem();
                        $returnArray["success"]       = true;
                        $returnArray["message"]       = ($returnArray["message"] != "") ? $returnArray["message"] : __("Your Account has been successfully created");
                        $returnArray["cartCount"]     = $quote->getItemsQty() * 1;
                        $returnArray["customerEmail"] = $email;
                        $returnArray["customerToken"] = $this->_helper->getTokenByCustomerDetails($email, $password, $customerId);
                        $this->_tokenHelper->saveToken($customerId, $token, $os);
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $returnArray["message"] = __($e->getMessage());
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            } catch (\Exception $e) {
                $returnArray["message"] = __($e->getMessage());
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }