<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class ForgotPassword extends AbstractCustomer   {

        public function execute()   {
            $returnArray                 = [];
            $returnArray["success"]      = false;
            $returnArray["message"]      = "";
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $email          = $wholeData["email"]     ?? "";
                        $storeId        = $wholeData["storeId"]   ?? 1;
                        $websiteId      = $wholeData["websiteId"] ?? 0;
                        $environment    = $this->_emulate->startEnvironmentEmulation($storeId);
                        $emailValidator = new \Zend\Validator\EmailAddress();
                        if (!$emailValidator->isValid($email)) {
                            $returnArray["message"] = __("Invalid email address.");
                            return $this->getJsonResponse($returnArray);
                        }
                        $customer = $this->_customerFactory->create()->setWebsiteId($websiteId)->loadByEmail($email);
                        if ($customer->getId() > 0) {
                            try {
                                $this->_accountManagement->initiatePasswordReset(
                                    $email,
                                    \Magento\Customer\Model\AccountManagement::EMAIL_REMINDER,
                                    $customer->getWebsiteId()
                                );
                                $returnArray["success"] = true;
                                $returnArray["message"] = __(
                                    "If there is an account associated with %1 you will receive an email with a link to reset your password.",
                                    $email
                                );
                            } catch (\Exception $e) {
                                $returnArray["message"] = $e->getMessage();
                            }
                        } else {
                            $returnArray["message"] = sprintf(__("Too many password reset requests. Please wait and try again or contact %1.", $this->_securityConfig->getCustomerServiceEmail()));
                        }
                        
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = __($e->getMessage());
                $this->_helper->printLog($e, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }