<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class MyDownloadsList extends AbstractCustomer      {

        public function execute()   {
            $returnArray                  = [];
            $returnArray["success"]       = false;
            $returnArray["message"]       = "";
            $returnArray["otherError"]    = "";
            $returnArray["totalCount"]    = 0;
            $returnArray["downloadsList"] = [];
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $storeId       = $wholeData["storeId"]       ?? 1;
                        $pageNumber    = $wholeData["pageNumber"]    ?? 1;
                        $customerToken = $wholeData["customerToken"] ?? "";
                        $environment   = $this->_emulate->startEnvironmentEmulation($storeId);
                        $customerId    = $this->_helper->getCustomerByToken($customerToken);
// checking customer token //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if (!$customerId && $customerToken != "")  {
                            $returnArray["otherError"] = "customerNotExist";
                            throw new \Magento\Framework\Exception\LocalizedException(
                                __("As customer you are requesting does not exist, so you need to logout.")
                            );
                        }
// end checking customer token ////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
                        $purchased    = $this->_purchasedLinkCollection->addFieldToFilter("customer_id", $customerId)->addOrder("created_at", "DESC");
                        $purchasedIds = [];
                        foreach ($purchased as $item)
                            $purchasedIds[] = $item->getId();
                        if (empty($purchasedIds))
                            $purchasedIds = [null];
                        $purchasedItems   = $this->_purchasedLinkItemCollection
                            ->addFieldToFilter("purchased_id", ["in"=>$purchasedIds])
                            ->addFieldToFilter("status", ["nin"=>[\Magento\Downloadable\Model\Link\Purchased\Item::LINK_STATUS_PENDING_PAYMENT, \Magento\Downloadable\Model\Link\Purchased\Item::LINK_STATUS_PAYMENT_REVIEW]])
                            ->setOrder("item_id", "desc");
// Applying pagination //////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if($pageNumber >= 1) {
                            $returnArray["totalCount"] = $purchasedItems->getSize();
                            $pageSize = $this->_helperCatalog->getPageSize();
                            $purchasedItems->setPageSize($pageSize)->setCurPage($pageNumber);
                        }
                        foreach ($purchasedItems as $item)
                            $item->setPurchased($purchased->getItemById($item->getPurchasedId()));
// Creating Downloads List //////////////////////////////////////////////////////////////////////////////////////////////////////
                        $downloadsList = [];
                        $block = $this->_listProduct;
                        foreach ($purchasedItems as $downloads) {
                            $eachDownloads                       = [];
                            $eachDownloads["incrementId"]        = $incrementId = $downloads->getPurchased()->getOrderIncrementId();
                            $order = $this->_order->loadByIncrementId($incrementId);
                            if ($order->getRealOrderId() > 0){
                                $eachDownloads["isOrderExist"]   = true;
                                $eachDownloads["message"]        = "";
                            }
                            else {
                                $eachDownloads["isOrderExist"]   = false;
                                $eachDownloads["message"]        = __("Sorry This Order Does not Exist!!");
                            }
                            $eachDownloads["hash"]               = $downloads->getLinkHash();
                            $eachDownloads["date"]               = $block->formatDate($downloads->getPurchased()->getCreatedAt());
                            $eachDownloads["proName"]            = $this->_helperCatalog->stripTags($downloads->getPurchased()->getProductName());
                            $eachDownloads["status"]             = __(ucfirst($downloads->getStatus()));
                            $eachDownloads["state"]              = $order->getState();
                            $eachDownloads["remainingDownloads"] = $block->getRemainingDownloads($downloads);
                            $canReorder                          = false;
                            if($this->canReorder($order))
                                $canReorder                      = $this->canReorder($order);
                            $eachDownloads["canReorder"]         = $canReorder;
                            $downloadsList[]                     = $eachDownloads;
                        }
                        $returnArray["downloadsList"]            = $downloadsList;
                        $returnArray["success"]                  = true;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $returnArray["message"] = __($e->getMessage());
                return $this->getJsonResponse($returnArray);    
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }