<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class OrderList extends AbstractCustomer        {

        public function execute()   {
            $returnArray                 = [];
            $returnArray["message"]      = "";
            $returnArray["otherError"]   = "";
            $returnArray["orderList"]    = [];
            $returnArray["totalCount"]   = 0;
            try {
                $wholeData = $this->getRequest()->getPostValue();
                $this->_headers   = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $storeId       = $wholeData["storeId"]       ?? 1;
                        $pageNumber    = $wholeData["pageNumber"]    ?? 1;
                        $customerToken = $wholeData["customerToken"] ?? "";
                        $customerId    = $this->_helper->getCustomerByToken($customerToken);
// checking customer token //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if (!$customerId && $customerToken != "")  {
                            $returnArray["otherError"] = "customerNotExist";
                            throw new \Magento\Framework\Exception\LocalizedException(
                                __("As customer you are requesting does not exist, so you need to logout.")
                            );
                        }
// end checking customer token ////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $orderCollection = $this->_orderCollection
                            ->addFieldToSelect("*")
                            ->addFieldToFilter("customer_id", $customerId)
                            ->addFieldToFilter("status", ["in"=>$this->_orderConfig->getVisibleOnFrontStatuses()])
                            ->setOrder("created_at", "DESC");
// Applying pagination //////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if($pageNumber >= 1) {
                            $returnArray["totalCount"] = $orderCollection->getSize();
                            $pageSize = $this->_helperCatalog->getPageSize();
                            $orderCollection->setPageSize($pageSize)->setCurPage($pageNumber);
                        }
// Creating Order List //////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $orderList = [];
                        foreach ($orderCollection as $key=>$order) {
                            $eachOrder = [];
                            $eachOrder["id"]          = $key;
                            $eachOrder["order_id"]    = $order->getRealOrderId();
                            $eachOrder["date"]        = $this->_orderHistoryBlock->formatDate($order->getCreatedAt());
                            $eachOrder["ship_to"]     = $order->getShippingAddress() ? $this->_helperCatalog->stripTags($order->getShippingAddress()->getName()) : " ";
                            $eachOrder["order_total"] = $this->_helperCatalog->stripTags($order->formatPrice($order->getGrandTotal()));
                            $eachOrder["state"]       = $order->getState();
                            $eachOrder["status"]      = $order->getStatusLabel();
                            $canReorder               = false;
                            if($this->canReorder($order))
                                $canReorder           = $this->canReorder($order);
                            $eachOrder["canReorder"]  = $canReorder;
                            $orderList[]              = $eachOrder;
                        }
                        $returnArray["orderList"]     = $orderList;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $returnArray["message"] = __($e->getMessage());
                return $this->getJsonResponse($returnArray);    
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }