<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class ReviewDetails extends AbstractCustomer      {

        public function execute()   {
            $returnArray                  = [];
            $returnArray["name"]          = "";
            $returnArray["image"]         = "";
            $returnArray["rating"]        = 0;
            $returnArray["message"]       = "";
            $returnArray["productId"]     = 0;
            $returnArray["ratingData"]    = [];
            $returnArray["reviewDate"]    = "";
            $returnArray["reviewDetail"]  = "";
            $returnArray["dominantColor"] = "";
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $width       = $wholeData["width"]    ?? 10000;
                        $storeId     = $wholeData["storeId"]  ?? 1;
                        $reviewId    = $wholeData["reviewId"] ?? 0;
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $review      = $this->_review->load($reviewId);
                        $product     = $this->_productFactory->create()->setStoreId($storeId)->load($review->getEntityPkValue());
                        $returnArray["productId"] = $product->getId();
                        $returnArray["name"]      = $this->_helperCatalog->stripTags($product->getName());
                        $returnArray["image"]     = $this->_helperCatalog->getImageUrl($product, $width/2);
                        $ratingArray              = [];
                        $ratingCollection         = $this->_vote
                            ->getResourceCollection()
                            ->setReviewFilter($reviewId)
                            ->addRatingInfo($storeId)
                            ->setStoreFilter($storeId)
                            ->load();
                        foreach ($ratingCollection as $rating) {
                            $eachRating                = [];
                            $eachRating["ratingCode"]  = $this->_helperCatalog->stripTags($rating->getRatingCode());
                            $eachRating["ratingValue"] = number_format($rating->getPercent(), 2, ".", "");
                            $ratingArray[]             = $eachRating;
                        }
                        $returnArray["ratingData"] = $ratingArray;
                        $returnArray["reviewDate"] = __("Your Review (submitted on ").$this->_helperCatalog->formatDate($review->getCreatedAt(), "long").")";
                        $returnArray["reviewDetail"] = $this->_helperCatalog->stripTags($review->getDetail());
                        $returnArray["dominantColor"] = $this->_helper->getDominantColor($returnArray["image"]);
                        $reviews = $this->_review
                            ->getResourceCollection()
                            ->addStoreFilter($storeId)
                            ->addEntityFilter("product", $product->getId())
                            ->addStatusFilter(\Magento\Review\Model\Review::STATUS_APPROVED)
                            ->setDateOrder()
                            ->addRateVotes();
                        $ratings = [];
                        if (count($reviews) > 0) {
                            foreach ($reviews->getItems() as $review) {
                                foreach ($review->getRatingVotes() as $vote) {
                                    $ratings[] = $vote->getPercent();
                                }
                            }
                        }
                        if (count($ratings) > 0)
                            $returnArray["rating"] = number_format((5*(array_sum($ratings) / count($ratings)))/100, 2, ".", "");
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }