<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class Wishlist extends AbstractCustomer  {

        public function execute()   {
            $returnArray               = [];
            $returnArray["success"]    = false;
            $returnArray["message"]    = "";
            $returnArray["wishList"]   = [];
            $returnArray["totalCount"] = 0;
            $returnArray["otherError"] = "";
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $width         = $wholeData["width"]         ?? 1000;
                        $storeId       = $wholeData["storeId"]       ?? 1;
                        $pageNumber    = $wholeData["pageNumber"]    ?? 1;
                        $customerToken = $wholeData["customerToken"] ?? "";
                        $customerId    = $this->_helper->getCustomerByToken($customerToken);
// checking customer token //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if (!$customerId && $customerToken != "")  {
                            $returnArray["otherError"] = "customerNotExist";
                            throw new \Magento\Framework\Exception\LocalizedException(
                                __("As customer you are requesting does not exist, so you need to logout.")
                            );
                        }
// end checking customer token ////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $wishlist    = $this->_wishlistProvider->loadByCustomerId($customerId, true);
                        $wishListCollection = $wishlist->getItemCollection();
// Applying pagination //////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if($pageNumber >= 1) {
                            $returnArray["totalCount"] = $wishListCollection->getSize();
                            $pageSize = $this->_helperCatalog->getPageSize();
                            $wishListCollection->setPageSize($pageSize)->setCurPage($pageNumber);
                        }
// Creating Wish List ///////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $wishList = [];
                        foreach ($wishListCollection as $item) {
                            $eachWishData = [];
                            $eachWishData["id"]          = $item->getId();
                            $product                     = $this->_productFactory->create()->load($item->getProductId());
                            $eachWishData["sku"]         = $product->getSku();
                            $eachWishData["qty"]         = $item->getQty() * 1;
                            $eachWishData["name"]        = $product->getName();
                            $eachWishData["price"]       = $this->_helperCatalog->stripTags($this->_pricingHelper->currency($product->getFinalPrice()));
                            $eachWishData["productId"]   = $product->getId();
                            $eachWishData["description"] = $item->getDescription();
                            $options                     = $this->_productConfigurationHelper->getOptions($item);
                            $eachWishData["options"]     = [];
                            if (count($options) > 0){
                                foreach ($options as $option){
                                    $eachOption                = [];
                                    $eachOption["label"]       = html_entity_decode($option["label"]);
                                    if (is_array($option["value"]))
                                        $eachOption["value"]   = $option["value"]; 
                                    else
                                        $eachOption["value"][] = $this->_helperCatalog->stripTags($option["value"]); 
                                    $eachWishData["options"][] = $eachOption;
                                }
                            }
                            $productData = $this->_helperCatalog->getOneProductRelevantData($product, $storeId, $width, $customerId);
                            $wishList[]  = array_merge($eachWishData, $productData);
                        }
                        $returnArray["success"]  = true;
                        $returnArray["wishList"] = $wishList;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $returnArray["message"] = __($e->getMessage());
                return $this->getJsonResponse($returnArray);    
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }