<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;
    use Magento\Framework\Exception\LocalizedException;

    class WishlistToCart extends AbstractCustomer    {

        public function execute()   {
            $returnArray               = [];
            $returnArray["success"]    = false;
            $returnArray["message"]    = "";
            $returnArray["cartCount"]  = 0;
            $returnArray["otherError"] = "";
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $qty         = $wholeData["qty"]        ?? 1;
                        $itemId      = $wholeData["itemId"]     ?? 0;
                        $storeId     = $wholeData["storeId"]    ?? 1;
                        $productId   = $wholeData["productId"]  ?? 0;
                        $customerToken = $wholeData["customerToken"] ?? "";
                        $customerId    = $this->_helper->getCustomerByToken($customerToken);
// checking customer token //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if (!$customerId && $customerToken != "")  {
                            $returnArray["otherError"] = "customerNotExist";
                            throw new \Magento\Framework\Exception\LocalizedException(
                                __("As customer you are requesting does not exist, so you need to logout.")
                            );
                        }
// end checking customer token ////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $item = $this->_wishlistItem->load($itemId);
                        if (!$item || !$item->getId())
                            throw new LocalizedException(__("Item id is invalid"));
                        $wishlist = $this->_wishlistProvider->load($item->getWishlistId());
                        if ($wishlist->getCustomerId() != $customerId)
                            throw new LocalizedException(__("Invalid data."));
                        $options = $this->_wishlistItemOption->getCollection()->addItemFilter([$itemId]);
                        $item->setQty($qty);
                        $item->setOptions($options->getOptionsByItem($itemId));
                        $buyRequest = $this->_productHelper->addParamsToBuyRequest(
                            ["item"=>$itemId, "qty"=>[$itemId=>$qty]],
                            ["current_config"=>$item->getBuyRequest()]
                        );
                        $item->mergeBuyRequest($buyRequest);
                        $quoteCollection = $this->_quote->getCollection()
                            ->addFieldToFilter("customer_id", $customerId)
                            ->addFieldToFilter("is_active", 1)
                            ->addOrder("updated_at", "DESC");
                        $quote = $quoteCollection->getFirstItem();
                        $quoteId = $quote->getId();
                        if ($quote->getId() < 0 || !$quoteId) {
                            $quote = $this->_quoteFactory->create()
                                ->setStoreId($storeId)
                                ->setIsActive(true)
                                ->setIsMultiShipping(false)
                                ->save();
                            $quoteId = (int) $quote->getId();
                            $customer = $this->_customerRepositoryInterface->getById($customerId);
                            $quote->assignCustomer($customer);
                            $quote->setCustomer($customer);
                            $quote->getBillingAddress();
                            $quote->getShippingAddress()->setCollectShippingRates(true);
                            $quote->collectTotals()->save();
                        }
                        $quote = $this->_quoteFactory->create()->setStoreId($storeId)->load($quoteId);
                        $this->_cart->setQuote($quote);
                        try {
                            $item->addToCart($this->_cart, true);
                        } catch (\Exception $e) {
                            $message = __($e->getMessage());
                            if (strlen($message) > 88) {
                                $message = substr($message, 0, 88)."...";
                            }
                            $returnArray["message"] = $message;
                            $this->_helper->log($returnArray, "logResponse", $wholeData);
                            return $this->getJsonResponse($returnArray);
                        }
                        $this->_cart->save()->getQuote()->collectTotals()->save();
                        $wishlist->save();
                        $returnArray["cartCount"] = $this->_cart->getQuote()->getItemsQty() * 1;
                        $returnArray["success"] = true;
                        $returnArray["message"] = __("Product(s) has successfully moved to cart.");
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $this->_helper->printLog($e);
                $returnArray["message"] = $e->getCustomMessage();
                return $this->getJsonResponse($returnArray);
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray);
                return $this->getJsonResponse($returnArray);
            }
        }

    }