<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Productalert;

    class Stock extends \Webkul\Mobikul\Controller\ApiController    {

        protected $_storeManager;
        protected $_productLoader;
        protected $_productAlertStock;
        protected $_emulate;

        public function __construct(
            \Webkul\Mobikul\Helper\Data $helper,
            \Magento\Store\Model\App\Emulation $emulate,
            \Webkul\Mobikul\Helper\Catalog $helperCatalog,
            \Magento\Framework\App\Action\Context $context,
            \Magento\ProductAlert\Model\Stock $productAlertStock,
            \Magento\Catalog\Model\ProductFactory $productLoader,
            \Magento\Store\Model\StoreManagerInterface $storeManager
        ) {
            $this->_storeManager      = $storeManager;
            $this->_productLoader     = $productLoader;
            $this->_productAlertStock = $productAlertStock;
            $this->_emulate           = $emulate;
            parent::__construct($helper, $context);
        }

        public function execute()   {
            $returnArray               = [];
            $returnArray["message"]    = "";
            $returnArray["success"]    = false;
            $returnArray["otherError"] = "";
            try {
                $wholeData           = $this->getRequest()->getPostValue();
                $this->_headers      = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey         = $this->getRequest()->getHeader("authKey");
                    $authData        = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $storeId       = $wholeData["storeId"]       ?? 1;
                        $productId     = $wholeData["productId"]     ?? 0;
                        $customerToken = $wholeData["customerToken"] ?? '';
                        $customerId    = $this->_helper->getCustomerByToken($customerToken) ?? 0;
// checking customer token ///////////////////////////////////////////////////////////////////////////////////////////////////////
                        if (!$customerId && $customerToken != "") {
                            $returnArray["message"]    = __("As customer you are requesting does not exist, so you need to logout.");
                            $returnArray["otherError"] = "customerNotExist";
                            $customerId = 0;           
                        }
// end checking customer token //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////                        
                        $environment   = $this->_emulate->startEnvironmentEmulation($storeId);
                        $product       = $this->_productLoader->create()->load($productId);
                        $model         = $this->_productAlertStock
                            ->setCustomerId($customerId)
                            ->setProductId($product->getId())
                            ->setPrice($product->getFinalPrice())
                            ->setWebsiteId($this->_storeManager->getStore()->getWebsiteId());
                        $model->save();
                        $returnArray["message"] = __("Alert subscription has been saved.");
                        $returnArray["success"] = true;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\NoSuchEntityException $noEntityException) {
                $returnArray["message"] = __("There are not enough parameters.");
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            } catch (\Exception $e) {
                $returnArray["message"] = __("We can't update the alert subscription right now.");
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }