<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Helper;
    use Magento\Framework\Stdlib\DateTime\DateTime;
    use Magento\Catalog\Api\CategoryRepositoryInterface;

    class Catalog extends \Magento\Framework\App\Helper\AbstractHelper  {

        protected $_date;
        protected $_directory;
        protected $_dateTime;
        protected $_customer;
        protected $_httpHeader;
        protected $_coreString;
        protected $_priceFormat;
        protected $mobikulHelper;
        protected $_storeManager;
        protected $_imageFactory;
        protected $_coreRegistry;
        protected $_catalogLayer;
        protected $_priceCurrency;
        protected $_layerResolver;
        protected $_objectManager;
        protected $_catalogHelper;
        protected $_stockRegistry;
        protected $_sessionManager;
        protected $_checkoutHelper;
        protected $_filterProvider;
        protected $_layerAttribute;
        protected $_categoryRepository;
        protected $_wishlistRepository;
        protected $_filterPriceDataprovider;

        public function __construct(
            DateTime $date,
            \Magento\Framework\Registry $coreRegistry,
            \Webkul\Mobikul\Helper\Data $mobikulHelper,
            \Magento\Catalog\Helper\Data $catalogHelper,
            \Magento\Checkout\Helper\Data $checkoutHelper,
            \Magento\Framework\App\Helper\Context $context,
            \Magento\Framework\Image\Factory $imageFactory,
            CategoryRepositoryInterface $categoryRepository,
            \Magento\Catalog\Model\Layer\Resolver $layerResolver,
            \Magento\Framework\Filesystem\DirectoryList $directory,
            \Magento\Framework\ObjectManagerInterface $objectManager,
            \Magento\Store\Model\StoreManagerInterface $storeManager,
            \Magento\Cms\Model\Template\FilterProvider $filterProvider,
            \Magento\Customer\Api\CustomerRepositoryInterface $customer,
            \Magento\Wishlist\Model\WishlistFactory $wishlistRepository,
            \Magento\Framework\Pricing\PriceCurrencyInterface $priceCurrency,
            \Magento\Framework\Session\SessionManagerInterface $sessionManager,
            \Magento\Catalog\Model\Layer\Filter\ItemFactory $filterItemFactory,
            \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
            \Magento\Catalog\Model\Layer\Filter\AttributeFactory $layerAttribute,
            \Magento\Catalog\Model\Layer\Filter\DataProvider\PriceFactory $filterPriceDataprovider
        )  {
            $this->_date                    = $date;
            $this->_baseDir                 = $directory->getPath("media");
            $this->_customer                = $customer;
            $this->_directory               = $directory;
            $this->_httpHeader              = $context->getHttpHeader();
            $this->_coreRegistry            = $coreRegistry;
            $this->_catalogLayer            = $layerResolver->get();
            $this->_storeManager            = $storeManager;
            $this->_imageFactory            = $imageFactory;
            $this->_objectManager           = $objectManager;
            $this->_catalogHelper           = $catalogHelper;
            $this->_priceCurrency           = $priceCurrency;
            $this->_stockRegistry           = $stockRegistry;
            $this->mobikulHelper            = $mobikulHelper;
            $this->_sessionManager          = $sessionManager;
            $this->_checkoutHelper          = $checkoutHelper;
            $this->_filterProvider          = $filterProvider;
            $this->_layerAttribute          = $layerAttribute;
            $this->_filterItemFactory       = $filterItemFactory;
            $this->_wishlistRepository      = $wishlistRepository;
            $this->_categoryRepository      = $categoryRepository;
            $this->_filterPriceDataprovider = $filterPriceDataprovider;
            parent::__construct($context);
            $this->_dateTime                = $this->_objectManager->create("\Magento\Framework\Stdlib\DateTime");
            $this->_coreString              = $this->_objectManager->create("\Magento\Framework\Stdlib\StringUtils");
            $this->_priceFormat             = $this->_objectManager->create("\Magento\Framework\Pricing\Helper\Data");
            $this->_storeSwitcher           = $this->_objectManager->create("\Magento\Store\Block\Switcher");
        }

        public function getCurrentStoreId()  {
            return $this->_storeManager->getStore()->getStoreId();
        }

        public function getAttributeInputType($attribute)  {
            $dataType  = $attribute->getBackend()->getType();
            $inputType = $attribute->getFrontend()->getInputType();
            if ($inputType == "select" || $inputType == "multiselect")
                return "select";
            elseif ($inputType == "boolean")
                return "yesno";
            elseif ($inputType == "price")
                return "price";
            elseif ($dataType == "int" || $dataType == "decimal")
                return "number";
            elseif ($dataType == "datetime")
                return "date";
            else
                return "string";
        }

        public function _renderRangeLabel($fromPrice, $toPrice, $storeId)  {
            $formattedFromPrice = $this->stripTags($this->_priceFormat->currency($fromPrice));
            if ($toPrice === "" || $toPrice < 1)  {
                return __("%1 and above", $formattedFromPrice);
            } elseif ($fromPrice == $toPrice && $this->scopeConfig->getValue("catalog/layered_navigation/one_price_interval", \Magento\Store\Model\ScopeInterface::SCOPE_STORE))  {
                return $formattedFromPrice;
            } else  {
                if ($fromPrice != $toPrice)
                    $toPrice -= .01;
                return __("%1 - %2", $formattedFromPrice, $this->stripTags($this->_priceFormat->currency($toPrice)));
            }
        }

        public function getPriceFilter($priceFilterModel, $storeId)  {
            if ($this->getPriceRangeCalculation() == "improved")  {
                $algorithmModel = $this->_objectManager->create("\Magento\Catalog\Model\Layer\Filter\Price\Algorithm");
                $collection = $priceFilterModel->getLayer()->getProductCollection();
                $appliedInterval = $priceFilterModel->getInterval();
                if ($appliedInterval && $collection->getPricesCount() <= $priceFilterModel->getIntervalDivisionLimit())
                    return [];
                $algorithmModel->setPricesModel($priceFilterModel)->setStatistics(
                    $collection->getMinPrice(),
                    $collection->getMaxPrice(),
                    $collection->getPriceStandardDeviation(),
                    $collection->getPricesCount()
                );
                if ($appliedInterval)  {
                    if ($appliedInterval[0] == $appliedInterval[1] || $appliedInterval[1] === "0")
                        return [];
                    $algorithmModel->setLimits($appliedInterval[0], $appliedInterval[1]);
                }
                $items = [];
                foreach ($algorithmModel->calculateSeparators() as $separator)  {
                    $items[] = [
                        "label" => $this->stripTags($this->_renderRangeLabel($separator["from"], $separator["to"], $storeId)),
                        "id"    => (($separator["from"] == 0) ? "" : $separator["from"])."-".$separator["to"].$priceFilterModel->_getAdditionalRequestData(),
                        "count" => $separator["count"]
                    ];
                }
            } elseif ($priceFilterModel->getInterval())  {
                return [];
            }
            $range    = $priceFilterModel->getPriceRange();
            $dbRanges = $priceFilterModel->getRangeItemCounts($range."");
            $data     = [];
            if (!empty($dbRanges))  {
                $lastIndex = array_keys($dbRanges);
                $lastIndex = $lastIndex[count($lastIndex) - 1];
                foreach ($dbRanges as $index => $count) {
                    $fromPrice = ($index == 1) ? "" : (($index - 1) * $range);
                    $toPrice   = ($index == $lastIndex) ? "" : ($index * $range);
                    $data[]    = [
                        "label" => $this->stripTags($this->_renderRangeLabel($fromPrice, $toPrice, $storeId)),
                        "id"    => $fromPrice."-".$toPrice,
                        "count" => $count
                    ];
                }
            }
            return $data;
        }

        public function getAttributeFilter($attributeFilterModel, $_filter)  {
            $options      = $_filter->getFrontend()->getSelectOptions();
            $optionsCount = $this->_objectManager
                ->create("\Magento\Catalog\Model\ResourceModel\Layer\Filter\Attribute")
                ->getCount($attributeFilterModel);
            $data = [];
            foreach ($options as $option)  {
                if (is_array($option["value"]))  {
                    continue;
                }
                if ($this->_coreString->strlen($option["value"]))  {
                    if ($_filter->getIsFilterable() == 1) {
                        if (!empty($optionsCount[$option["value"]]))  {
                            $data[] = [
                                "label" => html_entity_decode($option["label"]),
                                "id"    => $option["value"],
                                "count" => $optionsCount[$option["value"]]
                            ];
                        }
                    } else  {
                        $data[] = [
                            "label" => html_entity_decode($option["label"]),
                            "id"    => $option["value"],
                            "count" => isset($optionsCount[$option["value"]]) ? $optionsCount[$option["value"]] : 0
                        ];
                    }
                }
            }
            return $data;
        }

        public function getQueryArray($queryStringArray)  {
            $queryArray = [];
            foreach ($queryStringArray as $each)  {
                if (in_array($each["inputType"], ["string", "yesno"]))  {
                    if ($each["value"] != "")
                        $queryArray[$each["code"]] = $each["value"];
                } elseif (in_array($each["inputType"], ["price", "date"]))  {
                    $valueArray = $each["value"];
                    if ($valueArray["from"] != "" && $valueArray["to"] != "")
                        $queryArray[$each["code"]] = ["from"=>$valueArray["from"], "to"=>$valueArray["to"]];
                } elseif ($each["inputType"] == "select")  {
                    $valueArray = $each["value"];
                    $selectedArray = [];
                    foreach ($valueArray as $key=>$value)  {
                        if ($value == "true")
                            $selectedArray[] = $key;
                    }
                    if (count($selectedArray) > 0)
                        $queryArray[$each["code"]] = $selectedArray;
                }
            }
            return $queryArray;
        }

        public function getIfTaxIncludeInPrice()  {
            return $this->scopeConfig->getValue("tax/display/type", \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        }

        public function getOneProductRelevantData($product, $storeId, $width, $customerId=0)  {
            $this->_coreRegistry->unregister("current_product");
            $this->_coreRegistry->unregister("product");
            $this->_coreRegistry->register("current_product", $product);
            $this->_coreRegistry->register("product", $product);
            $reviews = $this->_objectManager->create("\Magento\Review\Model\Review")
                ->getResourceCollection()
                ->addStoreFilter($storeId)
                ->addEntityFilter("product", $product->getId())
                ->addStatusFilter(\Magento\Review\Model\Review::STATUS_APPROVED)
                ->setDateOrder()
                ->addRateVotes();
            $ratings = [];
            if (count($reviews) > 0) {
                foreach ($reviews->getItems() as $review)  {
                    foreach ($review->getRatingVotes() as $vote)
                        $ratings[] = $vote->getPercent();
                }
            }
            $isIncludeTaxInPrice = false;
            if ($this->getIfTaxIncludeInPrice() == 2)
                $isIncludeTaxInPrice = true;
            $eachProduct = [];
            if ($product->getTypeId() == "configurable")  {
                $configurableBlock               = $this->_objectManager->create("\Webkul\Mobikul\Block\Configurable");
                $eachProduct["configurableData"] = $configurableBlock->getJsonConfig();
            } else
                $eachProduct["configurableData"] = new \stdClass();
            $eachProduct["isInWishlist"]   = false;
            $eachProduct["wishlistItemId"] = 0;
            if($customerId != 0)  {
                $wishlist = $this->_wishlistRepository->create()->loadByCustomerId($customerId, true);
                $wishlistCollection = $this->_objectManager->create("Magento\Wishlist\Model\ResourceModel\Item\Collection")
                        ->addFieldToFilter("wishlist_id", $wishlist->getId())
                        ->addFieldToFilter("product_id", $product->getId());
                $item = $wishlistCollection->getFirstItem();
                $eachProduct["isInWishlist"] = !!$item->getId();
                if($eachProduct["isInWishlist"])
                    $eachProduct["wishlistItemId"] = (int)$item->getId();
            }
            $eachProduct["typeId"]   = $product->getTypeId();
            $eachProduct["entityId"] = $product->getId();
            if ($product->getTypeId() == "downloadable")  {
                $eachProduct["linksPurchasedSeparately"] = $product->getLinksPurchasedSeparately();
            }
            if ($product->getTypeId() == "bundle")  {
                $eachProduct["priceView"] = $product->getPriceView();
                $priceModel  = $product->getPriceModel();
                if ($isIncludeTaxInPrice)  {
                    list($minimalPriceInclTax, $maximalPriceInclTax) = $priceModel->getTotalPrices($product, null, true, false);
                    $eachProduct["minPrice"]         = $this->_catalogHelper->getTaxPrice($product, $minimalPriceInclTax);
                    $eachProduct["maxPrice"]         = $this->_catalogHelper->getTaxPrice($product, $maximalPriceInclTax);
                    $eachProduct["formatedMaxPrice"] = $this->stripTags($this->_priceFormat->currency($this->_catalogHelper->getTaxPrice($product, $maximalPriceInclTax)));
                    $eachProduct["formatedMinPrice"] = $this->stripTags($this->_priceFormat->currency($this->_catalogHelper->getTaxPrice($product, $minimalPriceInclTax)));
                } else  {
                    list($minimalPriceTax, $maximalPriceTax) = $priceModel->getTotalPrices($product, null, null, false);
                    $eachProduct["minPrice"]         = $minimalPriceTax;
                    $eachProduct["maxPrice"]         = $maximalPriceTax;
                    $eachProduct["formatedMinPrice"] = $this->stripTags($this->_priceFormat->currency($minimalPriceTax));
                    $eachProduct["formatedMaxPrice"] = $this->stripTags($this->_priceFormat->currency($maximalPriceTax));
                }
            }
            $eachProduct["shortDescription"] = html_entity_decode($product->getShortDescription());
            if($eachProduct["shortDescription"] == "")
                $eachProduct["shortDescription"] = html_entity_decode($product->getDescription());
            $rating = 0;
            if (count($ratings) > 0)
                $rating = number_format((5 * (array_sum($ratings) / count($ratings))) / 100, 2, ".", "");
            $eachProduct["rating"] = $rating;
            if ($product->isAvailable())
                $eachProduct["isAvailable"] = true;
            else
                $eachProduct["isAvailable"] = false;
            $price = $product->getPrice();
            if (!$price && $product->getTypeId() == "configurable")  {
                $regularPrice = $product->getPriceInfo()->getPrice('regular_price');
                $price = $regularPrice->getAmount()->getBaseAmount();
            } elseif(empty($price))
                $price = 0.0;
            if ($isIncludeTaxInPrice)  {
                $eachProduct["price"]                = $this->_catalogHelper->getTaxPrice($product, $price);
                $eachProduct["finalPrice"]           = $this->_catalogHelper->getTaxPrice($product, $product->getFinalPrice());
                $eachProduct["formatedPrice"]        = $this->stripTags($this->_priceFormat->currency($this->_catalogHelper->getTaxPrice($product, $price)));
                $eachProduct["formatedFinalPrice"]   = $this->stripTags($this->_priceFormat->currency($this->_catalogHelper->getTaxPrice($product, $product->getFinalPrice())));
            } else  {
                $eachProduct["price"]                = $price;
                $eachProduct["finalPrice"]           = $product->getFinalPrice();
                $eachProduct["formatedPrice"]        = $this->stripTags($this->_priceFormat->currency($price));
                $eachProduct["formatedFinalPrice"]   = $this->stripTags($this->_priceFormat->currency($product->getFinalPrice()));
            }
            $eachProduct["name"]                       = html_entity_decode($product->getName());
            $returnArray["msrpEnabled"]                = $product->getMsrpEnabled();
            $eachProduct["requiredOptions"]            = $product->getRequiredOptions();
            $returnArray["msrpDisplayActualPriceType"] = $product->getMsrpDisplayActualPriceType();
            if ($product->getTypeId() == "grouped")  {
                $minPrice = 0;
                if($product->getMinimalPrice() == "")  {
                    $associatedProducts = $product->getTypeInstance(true)->getAssociatedProducts($product);
                    $minPrice = [];
                    if (count($associatedProducts))  {
                        foreach ($associatedProducts as $associatedProduct)  {
                            if ($ogPrice = $associatedProduct->getPrice())
                                $minPrice[] = $ogPrice;
                        }
                    } else  {
                        $minPrice[] = 0;
                    }
                    $minPrice = min($minPrice);
                }
                else
                    $minPrice = $product->getMinimalPrice();
                if ($isIncludeTaxInPrice)
                    $eachProduct["groupedPrice"] = $this->stripTags($this->_priceFormat->currency($this->_catalogHelper->getTaxPrice($product, $minPrice)));
                else
                    $eachProduct["groupedPrice"] = $this->stripTags($this->_priceFormat->currency($minPrice));
            }
            $fromdate  = $product->getSpecialFromDate();
            $todate    = $product->getSpecialToDate();
            $isInRange = false;
            if (isset($fromdate) && isset($todate))  {
                $today     = $this->_date->date("Y-m-d H:i:s");
                $todayTime = strtotime($today);
                $fromTime  = strtotime($fromdate);
                $toTime    = strtotime($todate);
                if ($todayTime >= $fromTime && $todayTime <= $toTime)
                    $isInRange = true;
            }
            if (isset($fromdate) && !isset($todate))  {
                $today     = $this->_date->date("Y-m-d H:i:s");
                $todayTime = strtotime($today);
                $fromTime  = strtotime($fromdate);
                if ($todayTime >= $fromTime)
                    $isInRange = true;
            }
            if(!isset($fromdate) && isset($todate))  {
                $today      = $this->_date->date("Y-m-d H:i:s");
                $today_time = strtotime($today);
                $from_time  = strtotime($fromdate);
                if($today_time <= $from_time)
                    $isInRange = true;
            }
            $eachProduct["isInRange"] = $isInRange;
            $eachProduct["thumbNail"] = $this->getImageUrl($product, $width/2.5);
            $eachProduct["dominantColor"] = $this->mobikulHelper->getDominantColor($eachProduct["thumbNail"]);
            $stockItem = $this->_stockRegistry->getStockItem($product->getId(), $product->getStore()->getWebsiteId());
            $minQty = $stockItem->getMinSaleQty();
            $config = $product->getPreconfiguredValues();
            $configQty = $config->getQty();
            if ($configQty > $minQty)  {
                $minQty = $configQty;
            }
            $eachProduct["arUrl"]           = $this->getArModelUrl($product);
            $eachProduct["arType"]          = $this->getArModelType($product);
            $eachProduct["arTextureImages"] = $this->getTextureImages($product);
            $eachProduct["minAddToCartQty"] = $minQty;
            return $eachProduct;
        }

        public function getStoreData()  {
            $storeData                    = [];
            $storeBlock                   = $this->_storeSwitcher;
            foreach ($storeBlock->getGroups() as $group)  {
                $groupArr                 = [];
                $groupArr["id"]           = $group->getGroupId();
                $groupArr["name"]         = $group->getName();
                $stores                   = $group->getStores();
                foreach ($stores as $store)  {
                    if (!$store->isActive())
                        continue;
                    $storeArr             = [];
                    $storeArr["id"]       = $store->getStoreId();
                    $code                 = explode("_", $this->getLocaleCodes($store->getId()));
                    $storeArr["code"]     = $code[0];
                    $storeArr["name"]     = $store->getName();
                    $groupArr["stores"][] = $storeArr;
                }
                $storeData[]              = $groupArr;
            }
            return $storeData;
        }

        public function stripTags($data)  {
            return strip_tags($data);
        }

        public function getLocaleCodes($store)  {
            return $this->scopeConfig->getValue("general/locale/code", \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);
        }

        public function showOutOfStock()  {
            return $this->scopeConfig->getValue("cataloginventory/options/show_out_of_stock", \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        }

        public function getPageSize()  {
            return $this->scopeConfig->getValue("mobikul/configuration/pagesize", \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        }

        public function getPriceRangeCalculation()  {
            return $this->scopeConfig->getValue("catalog/layered_navigation/price_range_calculation", \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        }

        public function getMaxQueryLength()  {
            return $this->scopeConfig->getValue(\Magento\Search\Model\Query::XML_PATH_MAX_QUERY_LENGTH, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        }

        public function formatDate($date, $format = null)  {
            if ($format != null)
                return $this->_dateTime->formatDate($date, $format);
            else
                return $this->_dateTime->formatDate($date);
        }

        public function escapeHtml($text)  {
            return $this->_objectManager->create("\Magento\Framework\Escaper")->escapeHtml($text);
        }

        public function getBasePath($folder = "media")  {
            return $this->_directory->getPath($folder);
        }

        public function getImageUrl($product, $size, $imageType="product_page_image_small", $keepFrame=true)  {
            return $this->_objectManager
                ->create("\Magento\Catalog\Helper\Image")
                ->init($product, $imageType)
                ->keepFrame($keepFrame)
                ->resize($size)
                ->getUrl();
        }

        public function resizeNCache($basePath, $newPath, $width, $height, $forCustomer=false)  {
            if (!is_file($newPath) || $forCustomer)  {
                $imageObj = $this->_imageFactory->create($basePath);
                $imageObj->keepAspectRatio(false);
                $imageObj->backgroundColor([255, 255, 255]);
                $imageObj->keepFrame(false);
                $imageObj->resize($width, $height);
                $imageObj->save($newPath);
            }
        }

        public function getPriceFilterOptions($filter, $collection)  {
            try  {
                $facets = $collection->getFacetedData($filter->getAttributeCode());
            } catch (\Exception $e)  {
                return [];
            }
            $productSize = $collection->getSize();
            $data = [];
            if (count($facets) > 1)  {
                foreach ($facets as $key => $aggregation)  {
                    $count = $aggregation["count"];
                    if (strpos($key, "_") === false)  {
                        continue;
                    }
                    if (!$this->isOptionReducesResults($count, $productSize))  {
                        continue;
                    }
                    $data[] = $this->prepareData($key, $count, $data);
                }
            }
            return $data;
        }

        public function getFilterOptions($filter, $productCollection)  {
            $allData = [];
            $attributeFilterModel = $this->_layerAttribute->create()->setAttributeModel($filter);
            try  {
                $optionsFacetedData = $productCollection->getFacetedData($filter->getAttributeCode());
            } catch (\Exception $e)  {
                return [];
            }
            $options = $filter->getFrontend()->getSelectOptions();
            $productSize = $productCollection->getSize();
            foreach ($options as $option)  {
                $value = $this->getOptionValue($option);
                if ($value === false)
                    continue;
                $isAttributeFilterable = (int)$filter->getIsFilterable() === 1;
                $count = $this->getOptionCount($value, $optionsFacetedData);
                if ($isAttributeFilterable && (!$this->isOptionReducesResults($count, $productSize) || $count === 0))
                    continue;
                $each["id"]    = $value;
                $each["label"] = $option["label"];
                $each["count"] = $count;
                $allData[]     = $each;
            }
            return $allData;
        }

        public function getProductListColl($categoryId, $type = null)  {
            if ($type == "search")
                $layer = $this->_objectManager->create("\Magento\Catalog\Model\Layer\Search");
            else
                $layer = $this->_catalogLayer;
            $origCategory = null;
            if ($categoryId) {
                try {
                    $category = $this->_categoryRepository->get($categoryId);
                } catch (\NoSuchEntityException $e) {
                    $category = null;
                }
                if ($category)  {
                    $origCategory = $layer->getCurrentCategory();
                    $layer->setCurrentCategory($category);
                }
            }
            $collection = $layer->getProductCollection();
            if ($origCategory)  {
                $layer->setCurrentCategory($origCategory);
            }
            return $collection;
        }

        public function getOptionValue($option)  {
            if (empty($option["value"]) && !is_numeric($option["value"]))  {
                return false;
            }
            return $option["value"];
        }

        public function getOptionCount($value, $optionsFacetedData)  {
            return $optionsFacetedData[$value]["count"] ?? 0;
        }

        public function isOptionReducesResults($optionCount, $totalSize)  {
            return $optionCount < $totalSize;
        }

        public function _createItem($label, $value, $count = 0)  {
            return $this->_filterItemFactory->create()
                ->setFilter($this)
                ->setLabel($label)
                ->setValue($value)
                ->setCount($count);
        }

        private function prepareData($key, $count)  {
            list($from, $to) = explode("_", $key);
            if ($from == "*") {
                $from = $this->getFrom($to);
            }
            if ($to == "*") {
                $to = $this->getTo($to);
            }
            $label = $this->_renderPriceRangeLabel($from, $to);
            $value = $from . "-" . $to . $this->_filterPriceDataprovider->create()->getAdditionalRequestData();
    
            $data = [
                "id"    => $value,
                "label" => $label,
                "count" => $count
            ];
    
            return $data;
        }

        protected function _renderPriceRangeLabel($fromPrice, $toPrice)  {
            $fromPrice = empty($fromPrice) ? 0 : $fromPrice * $this->getCurrencyRate();
            $toPrice = empty($toPrice) ? $toPrice : $toPrice * $this->getCurrencyRate();
            $formattedFromPrice = $this->stripTags($this->_priceCurrency->format($fromPrice));
            if ($toPrice === "")  {
                return __("%1 and above", $formattedFromPrice);
            } elseif ($fromPrice == $toPrice && $this->_filterPriceDataprovider->create()->getOnePriceIntervalValue()) {
                return $formattedFromPrice;
            } else  {
                if ($fromPrice != $toPrice)  {
                    $toPrice -= .01;
                }
                return __("%1 - %2", $formattedFromPrice, $this->stripTags($this->_priceCurrency->format($toPrice)));
            }
        }

        public function getTo($from)  {
            $to = "";
            $interval = $this->_filterPriceDataprovider->create()->getInterval();
            if ($interval && is_numeric($interval[1]) && $interval[1] > $from)  {
                $to = $interval[1];
            }
            return $to;
        }

        public function getCurrencyRate()  {
            $rate = $this->_storeManager->getStore()
                ->getCurrentCurrencyRate();
            if (!$rate)  {
                $rate = 1;
            }
            return $rate;
        }

        public function getFrom($from)  {
            $to = "";
            $interval = $this->_filterPriceDataprovider->create()->getInterval();
            if ($interval && is_numeric($interval[0]) && $interval[0] < $from)  {
                $to = $interval[0];
            }
            return $to;
        }

        public function getUrl($dir)  {
            return $this->_storeManager->getStore()->getBaseUrl($dir);
        }

        public function getResizedBanner($type)
        {
            $height = 400;
            $width  = 500;
            $image  = $this->scopeConfig->getValue("mobikul/configuration/".$type, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            $basePath = $this->_baseDir.DS."mobikul".DS."banner".DS.$image;
            $newUrl = "";
            if (is_file($basePath))  {
                $newPath = $this->_baseDir.DS."mobikulresized".DS."banner".DS.$width."x".$height.DS.$image;
                $this->resizeNCache($basePath, $newPath, $width, $height);
                $newUrl = $this->getUrl("media")."mobikulresized".DS."banner".DS.$width."x".$height.DS.$image;
            }
            return $newUrl;
        }

        public function getArModelType($product)
        {
            if ($product->getArType() == 1) {
                return "3D";
            }
            return "2D";
        }

        public function getArModelUrl($product)
        {
            $userAgent = $this->_httpHeader->getHttpUserAgent();
            if ($product->getArType() == 1 && strpos(strtolower($userAgent), "alamofire") !== false) {
                return $product->getArModelFileIos();
            } elseif ($product->getArType() == 1) {
                return $product->getArModelFileAndroid();
            }
            return $product->getAr2dFile();
        }

        public function getTextureImages($product)
        {
            $textureImages = [];
            $userAgent = $this->_httpHeader->getHttpUserAgent();
            if ($product->getArType() == 1 && $product->getArTextureImage() && strpos(strtolower($userAgent), "alamofire") !== false) {
                $textureImages = array_values((array)json_decode($product->getArTextureImage()));
            }
            return $textureImages;
        }
    }
