<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Helper;

    class Data extends \Magento\Framework\App\Helper\AbstractHelper  {

        protected $file;
        protected $_token;
        protected $jsonHelper;
        protected $_encrypted;
        protected $_storeManager;
        protected $_tokenService;
        protected $_directoryList;
        protected $_sessionManager;

        public function __construct(
            \Magento\Integration\Model\Oauth\Token $token,
            \Magento\Framework\App\Helper\Context $context,
            \Magento\Framework\Filesystem\Driver\File $file,
            \Magento\Framework\Json\Helper\Data $jsonHelper,    
            \Magento\Store\Model\StoreManagerInterface $storeManager,
            \Magento\Config\Model\Config\Backend\Encrypted $encrypted,
            \Magento\Framework\Filesystem\DirectoryList $directoryList,
            \Magento\Integration\Model\CustomerTokenService $tokenService,
            \Magento\Framework\Session\SessionManagerInterface $sessionManager
        )  {
            $this->file            = $file;
            $this->_token          = $token;
            $this->_encrypted      = $encrypted;
            $this->jsonHelper      = $jsonHelper;
            $this->_storeManager   = $storeManager;
            $this->_tokenService   = $tokenService;
            $this->_directoryList  = $directoryList;
            $this->_sessionManager = $sessionManager;
            parent::__construct($context);
        }

        public function getUrl($dir)  {
            return $this->_storeManager->getStore()->getBaseUrl($dir);
        }

        public function isAuthorized($authKey)  {
            $authData          = [];
            $authData["code"]  = 2;
            $authData["token"] = "";
            $apiKey       = $this->getPassword();
            $apiUsername  = $this->getConfigData("mobikul/configuration/apiusername");
            $sessionToken = $this->_sessionManager->getSessionId();
            $H1 = md5($apiUsername.":".$apiKey);
            $H2 = md5($H1.":".$sessionToken);
            if($authKey == $H2)
                $authData["code"] = 1;
            else
                $authData["token"] = $sessionToken;
            return $authData;
        }

        public function log($data, $key, $wholeData)  {
            $flag = $wholeData[$key] ?? 0;
            $this->printLog($data, $flag);
        }

        public function printLog($data, $flag=1, $filename="mobikul.log")  {
            if($flag == 1)  {
                $path   = $this->_directoryList->getPath("var");
                $logger = new \Zend\Log\Logger();
                if (!file_exists($path."/log/"))
                    mkdir($path."/log/", 0777, true);
                $logger->addWriter(new \Zend\Log\Writer\Stream($path."/log/".$filename));
                if (is_array($data) || is_object($data))
                    $data = print_r($data, true);
                $logger->info($data);
            }
        }

        public function validate($wholeData, $key)  {
            if(isset($wholeData[$key]) && $wholeData[$key] != "")
                return $wholeData[$key];
            else
                return false;
        }

        public function getConfigData($path, $scope=\Magento\Store\Model\ScopeInterface::SCOPE_STORE)  {
            $storeId = $this->_storeManager->getStore()->getId();
            return $this->scopeConfig->getValue($path, $scope, $storeId);
        }

        public function canReorder(\Magento\Sales\Model\Order $order)  {
            if (!$this->getConfigData("sales/reorder/allow"))
                return 0;
            if (1)
                return $order->canReorder();
            else
                return 1;
        }

        public function getPassword()  {
            return $this->_encrypted->processValue(
                $this->scopeConfig->getValue("mobikul/configuration/apikey", \Magento\Store\Model\ScopeInterface::SCOPE_STORE)
            );
        }

        public function getTokenByCustomerDetails($username, $password, $customerId)  {
            $token = $this->_token->loadByCustomerId($customerId)->getToken();
            if (!$token && $username && $password)
                $token = $this->createCustomerAccessToken($username, $password);
            return $token;
        }

        public function createCustomerAccessToken($username, $password)  {
            return $this->_tokenService->createCustomerAccessToken($username, $password);
        }

        public function getCustomerByToken($token)  {
            return $this->_token->loadByToken($token)->getCustomerId();
        }

        public function getDecodedText($string)
        {
            $allemojies = explode ('"', $string);
            $finalString = "";
            foreach ($allemojies as $emoji) {
                if (strpos($emoji, "\u") !== false ) {
                    $finalString = $finalString.json_decode('"'.$emoji.'"');
                } else {
                    $finalString = $finalString.$emoji;
                }
            }
            return $finalString;
        }

        /**
         * Get dominant color from saved file
         *
         * @param string $filePath image path
         *
         * @return string Dominant color of the image
         */
        public function getDominantColor($filePath)
        {
            $path      = $this->_directoryList->getPath("pub");
            $sha1Tag   = sha1($filePath);
            $cacheFile = "color.json";
            if (!$this->file->isExists($path."/media/mobikul/")) {
                $this->file->createDirectory($path."/media/mobikul/");
            }
            $allColorTags = "[]";
            try {
                $allColorTags = $this->file->fileGetContents($path."/media/mobikul/".$cacheFile);
            } catch (\Magento\Framework\Exception\FileSystemException $e) {
            }
            $allColorTags = $this->jsonHelper->jsonDecode($allColorTags);
            if (empty($allColorTags[$sha1Tag])) {
                $allColorTags[$sha1Tag] = $this->getColor($filePath);
                try {
                    $this->file->filePutContents($path."/media/mobikul/".$cacheFile, $this->jsonHelper->jsonEncode($allColorTags));
                } catch (\Magento\Framework\Exception\FileSystemException $e) {
                }
            }
            return $allColorTags[$sha1Tag];
        }

        /**
         * Function getColor to get image color
         *
         * @param string $filePath file path
         *
         * @return string Dominant color of the image
         */
        public function getColor($filePath)
        {
            $total = $blueTotal = $greenTotal = $redTotal = 0;
            $imageInfo = @getimagesize($filePath);
            if (!empty($imageInfo['mime'])) {
                switch ($imageInfo['mime']) {
                    case 'image/jpeg':
                        $image_create_func = 'imagecreatefromjpeg';
                        break;
                    case 'image/png':
                        $image_create_func = 'imagecreatefrompng';
                        break;
                    case 'image/gif':
                        $image_create_func = 'imagecreatefromgif';
                        break;
                }
                $image = $image_create_func($filePath);
                for ($x=0; $x<@imagesx($image); $x++) {
                    for ($y=0; $y<@imagesy($image); $y++) {
                        $rgb = @imagecolorat($image, $x, $y);
                        $red = ($rgb >> 16) &0xFF;
                        $green = ($rgb >> 8) &0xFF;
                        $blue = $rgb & 0xFF;
                        $redTotal += $red;
                        $greenTotal += $green;
                        $blueTotal += $blue;
                        $total++;
                    }
                }
                $redAverage = round($redTotal/$total);
                $greenAverage = round($greenTotal/$total);
                $blueAverage = round($blueTotal/$total);
                return sprintf("#%02x%02x%02x", $redAverage, $greenAverage, $blueAverage);
            }
            return "#f6f6f6";
        }
    }
