<?php
/**
 * Webkul Software.
 *
 * @category  Webkul
 * @package   Webkul_Mobikul
 * @author    Webkul
 * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */
namespace Webkul\Mobikul\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Filesystem\DirectoryList;

class ProductSaveAfterObserver implements ObserverInterface
{
    /**
     * @var RequestInterface
     */
    protected $_request;

    /**
     * @var \Magento\Catalog\Model\ProductRepository
     */
    protected $_productRepository;

    /**
     * @var ObjectManagerInterface
     */
    protected $_objectManager;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\Action
     */
    protected $_productAction;

    /**
     *
     * @param RequestInterface                                   $request
     * @param \Magento\Catalog\Model\ProductRepository           $productRepository
     * @param \Magento\Framework\ObjectManagerInterface          $objectManager
     */
    public function __construct(
        RequestInterface $request,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Filesystem\Driver\File $file,
        \Magento\Catalog\Model\ProductRepository $productRepository,
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ResourceModel\Product\Action $productAction,
        \Magento\MediaStorage\Model\File\UploaderFactory $fileUploaderFactory
    ) {
        $this->_request = $request;
        $this->_file = $file;
        $this->_filesystem = $filesystem;
        $this->_fileUploaderFactory = $fileUploaderFactory;
        $this->_productRepository = $productRepository;
        $this->_objectManager = $objectManager;
        $this->_productAction = $productAction;
        $this->_storeManager = $storeManager;
    }

    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $product = $observer->getEvent()->getProduct();
        $productData = $this->_request->getParam('product');
        $files = $this->_request->getFiles();
        $deleteArray = $this->_request->getParam("texture_image_delete");
        $storeId = $this->_request->getParam("store");
        $path = $this->_filesystem->getDirectoryRead(
            DirectoryList::MEDIA
        )->getAbsolutePath(
            'catalog/product/ar/textureimage/'
        ).$product->getId();
        $textureImages = json_decode($product->getArTextureImage());
        if (!empty($deleteArray)) {
            $imageForDelete = [];
            $finalTextureImages = [];
            foreach ($deleteArray as $url => $delete) {
                if ($delete) {
                    foreach ($textureImages as $image) {
                        if (strpos($image, $url) !== false) {
                            $imageForDelete[] = $image;
                            $exploadedFileArr = explode("/", $image);
                            $fileName = $exploadedFileArr[count($exploadedFileArr) - 1];
                            if ($this->_file->isExists($path."/".$fileName))  {
                                $this->_file->deleteFile($path."/".$fileName);
                            }
                        }
                    }
                }
            }
            $finalTextureImages = array_diff((array)$textureImages, $imageForDelete);
        } else {
            $finalTextureImages = json_decode($product->getArTextureImage());
        }
        if (!empty($files["ar_texture_image"])) {
            $keys = array_keys($files["ar_texture_image"]);
            foreach ($keys as $value) {
                try {
                    /** @var $uploader \Magento\MediaStorage\Model\File\Uploader */
                    $uploader = $this->_fileUploaderFactory->create(['fileId' => 'ar_texture_image['.$value.']']);
                    $uploader->setAllowedExtensions(['png', 'jpg', 'jpeg']);
                    $uploader->setAllowRenameFiles(true);
                    $result = $uploader->save($path);
                    $url = $this->getMediaUrl()."catalog/product/ar/textureimage/".$product->getId()."/".$result['file'];
                    $finalTextureImages[] = $url;
                } catch (\Exception $e) {
                }
            }
        }
        $this->_productAction->updateAttributes([$product->getId()], ['ar_texture_image' => json_encode($finalTextureImages)], $storeId);
    }
    public function getMediaUrl()
    {
        return $this->_storeManager->getStore()
                ->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
    }
}
