<?php
/**
 * @category   Webkul
 * @package    Webkul_Mobikul
 * @author     Webkul Software Private Limited
 * @copyright  Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
 * @license    https://store.webkul.com/license.html
 */
namespace Webkul\Mobikul\Setup;

use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Eav\Model\Config;
use Magento\Eav\Model\Entity\Attribute\Set as AttributeSet;
use Magento\Catalog\Model\ResourceModel\Product as ResourceProduct;
use Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface;

class UpgradeData implements UpgradeDataInterface   {

    protected $_reader;
    protected $_eavConfig;
    protected $_fileSystem;
    protected $_attributeSet;
    protected $_filesystemFile;
    protected $_eavSetupFactory;
    protected $_resourceProduct;

    public function __construct(
        Config $eavConfig,
        AttributeSet $attributeSet,
        ResourceProduct $resourceProduct,
        EavSetupFactory $eavSetupFactory,
        \Magento\Framework\Filesystem $fileSystem,
        \Magento\Framework\Module\Dir\Reader $reader,
        \Magento\Framework\Filesystem\Io\File $filesystemFile
    ) {
        $this->_reader          = $reader;
        $this->_eavConfig       = $eavConfig;
        $this->_fileSystem      = $fileSystem;
        $this->_attributeSet    = $attributeSet;
        $this->_filesystemFile  = $filesystemFile;
        $this->_eavSetupFactory = $eavSetupFactory;
        $this->_resourceProduct = $resourceProduct;
    }

    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $eavSetup = $this->_eavSetupFactory->create(["setup" => $setup]);
        if (version_compare($context->getVersion(), '2.0.5', '<')) {
            $groupName = 'Mobikul Configuration';
            $entityTypeId = $eavSetup->getEntityTypeId('catalog_product');
            $attributeSetIds = $eavSetup->getAllAttributeSetIds($entityTypeId);
            foreach($attributeSetIds as $attributeSetId) {
                $eavSetup->addAttributeGroup($entityTypeId, $attributeSetId, $groupName, 2);
                $attributeGroupId = $eavSetup->getAttributeGroupId($entityTypeId, $attributeSetId, $groupName);
                $attributeId = $eavSetup->getAttributeId($entityTypeId, 'as_featured');
                $eavSetup->addAttributeToGroup($entityTypeId, $attributeSetId, $attributeGroupId, $attributeId, null);
            }
            $this->_eavConfig->getAttribute('catalog_product', 'as_featured')->setSortOrder(1)->save();
            $typeSource = 'Webkul\Mobikul\Model\Config\Source\ArOptions';

            $eavSetup->addAttribute(
                \Magento\Catalog\Model\Product::ENTITY,
                'ar_type',
                [
                    'group' => 'Mobikul Configuration',
                    'type' => 'varchar',
                    'backend' => '',
                    'frontend' => '',
                    'label' => 'Ar Model Type',
                    'input' => 'select',
                    'class' => '',
                    'source' => $typeSource,
                    'global' => ScopedAttributeInterface::SCOPE_GLOBAL,
                    'visible' => true,
                    'required' => false,
                    'user_defined' => false,
                    'default' => '',
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'visible_on_front' => false,
                    'used_in_product_listing' => false,
                    'unique' => false,
                    'sort_order' => 2,
                    'apply_to' => 'simple,configurable',
                ]
            );

            $eavSetup->addAttribute(
                \Magento\Catalog\Model\Product::ENTITY,
                'ar_2d_file',
                [
                    'group' => 'Mobikul Configuration',
                    'type' => 'varchar',
                    'label' => 'AR 2-D Model File',
                    'input' => 'file',
                    'backend' => 'Webkul\Mobikul\Model\Product\Attribute\Backend\Ar',
                    'frontend' => '',
                    'class' => '',
                    'source' => '',
                    'global' => ScopedAttributeInterface::SCOPE_GLOBAL,
                    'visible' => true,
                    'required' => false,
                    'user_defined' => true,
                    'default' => '',
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'visible_on_front' => false,
                    'unique' => false,
                    'apply_to' => 'simple,configurable',
                    'used_in_product_listing' => false,
                    'sort_order' => 3,
                    'note' => "Allowed file type: png."
                ]
            );

            $eavSetup->addAttribute(
                \Magento\Catalog\Model\Product::ENTITY,
                'ar_model_file_android',
                [
                    'group' => 'Mobikul Configuration',
                    'type' => 'varchar',
                    'label' => 'AR Model File For Android',
                    'input' => 'file',
                    'backend' => 'Webkul\Mobikul\Model\Product\Attribute\Backend\Ar',
                    'frontend' => '',
                    'class' => '',
                    'source' => '',
                    'global' => ScopedAttributeInterface::SCOPE_GLOBAL,
                    'visible' => true,
                    'required' => false,
                    'user_defined' => true,
                    'default' => '',
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'visible_on_front' => false,
                    'unique' => false,
                    'apply_to' => 'simple,configurable',
                    'used_in_product_listing' => false,
                    'sort_order' => 4,
                    'note' => "Allowed file type: sfb."
                ]
            );

            $eavSetup->addAttribute(
                \Magento\Catalog\Model\Product::ENTITY,
                'ar_model_file_ios',
                [
                    'group' => 'Mobikul Configuration',
                    'type' => 'varchar',
                    'label' => 'AR Model File For Ios',
                    'input' => 'file',
                    'backend' => 'Webkul\Mobikul\Model\Product\Attribute\Backend\Ar',
                    'frontend' => '',
                    'class' => '',
                    'source' => '',
                    'global' => ScopedAttributeInterface::SCOPE_GLOBAL,
                    'visible' => true,
                    'required' => false,
                    'user_defined' => true,
                    'default' => '',
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'visible_on_front' => false,
                    'unique' => false,
                    'apply_to' => 'simple,configurable',
                    'used_in_product_listing' => false,
                    'sort_order' => 5,
                    'note' => "Allowed file type: obj."
                ]
            );

            $eavSetup->addAttribute(
                \Magento\Catalog\Model\Product::ENTITY,
                'ar_texture_image',
                [
                    'group' => 'Mobikul Configuration',
                    'type' => 'text',
                    'label' => 'Texture Images',
                    'input' => 'file',
                    'backend' => '',
                    'frontend' => '',
                    'class' => '',
                    'source' => '',
                    'global' => ScopedAttributeInterface::SCOPE_GLOBAL,
                    'visible' => false,
                    'required' => false,
                    'user_defined' => false,
                    'default' => '',
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'visible_on_front' => false,
                    'unique' => false,
                    'apply_to' => 'simple,configurable',
                    'used_in_product_listing' => false,
                    'sort_order' => 6
                ]
            );

            $entityType = $this->_resourceProduct->getEntityType();
            $attributeSetCollection = $this->_attributeSet->setEntityTypeFilter($entityType);
            foreach ($attributeSetCollection as $attributeSet) {
                $attributes = ["ar_type", "ar_model_file_android", "ar_model_file_ios", "ar_texture_files"];
                foreach ($attributes as $attribute) {
                    $eavSetup->addAttributeToSet("catalog_product", $attributeSet->getAttributeSetName(), "General", $attribute);
                }
            }
        }
    }
}
