<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Pos
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Setup;
    
    use Magento\Framework\Setup\ModuleContextInterface;
    use Magento\Framework\Setup\UpgradeSchemaInterface;
    use Magento\Framework\Setup\SchemaSetupInterface;

    class UpgradeSchema implements UpgradeSchemaInterface   {

        public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)   {
            $textType            = \Magento\Framework\DB\Ddl\Table::TYPE_TEXT;
            $integerType         = \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER;
            $decimalType         = \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL;
            $smallintType        = \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT;
            $timestampType       = \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP;
            $timestampInit       = \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT;
            $timestampInitUpdate = \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT_UPDATE;
            $setup->startSetup();
            $setup->getConnection()->addColumn(
                $setup->getTable("mobikul_devicetoken"),
                "email", [
                    "type"     => $textType,
                    "unsigned" => true,
                    "nullable" => true,
                    "default"  => null,
                    "comment"  => "Email for guest user"
                ]
            );

            $setup->getConnection()->addColumn(
                $setup->getTable("mobikul_devicetoken"),
                "os", [
                    "type"     => $textType,
                    "unsigned" => true,
                    "nullable" => true,
                    "default"  => null,
                    "comment"  => "Operating System"
                ]
            );

            $setup->getConnection()->addColumn(
                $setup->getTable("mobikul_categoryimages"),
                "store_id", [
                    "type"     => $textType,
                    "unsigned" => true,
                    "nullable" => true,
                    "default"  => null,
                    "comment"  => "Store Id"
                ]
            );
            if (version_compare($context->getVersion(), '2.0.5', '<')) {
                $table = $setup->getConnection()
                ->newTable($setup->getTable("mobikul_sales_order"))
                ->addColumn("order_id", $textType, null, ["nullable"=>true, "default"=>null], "Real Order Id")
                ->addColumn("real_order_id", $textType, null, ["nullable"=>true, "default"=>null], "Real Order Id")
                ->addColumn("store_id", $integerType, null, ["unsigned"=>true, "nullable"=>false, "default"=>"0"], "Store Id")
                ->addColumn("id", $integerType, null, ["identity"=>true, "unsigned"=>true, "nullable"=>false, "primary"=>true], "Id")
                ->addColumn("customer_id", $integerType, null, ["unsigned"=>true, "nullable"=>false, "default"=>"0"], "Customer Id")
                ->addColumn("customer_name", $textType, null, ["nullable"=>false, "default"=>""], "Customer Name")
                ->addColumn("created_at", $timestampType, null, ["nullable"=>false, "default"=>$timestampInit], "Creation Date Time")
                ->addColumn("order_total", $decimalType, "12,4", [], "Order Total")
                ->setComment("Mobikul Sales Table");

                $setup->getConnection()->createTable($table);
            }

            $setup->endSetup();
        }

    }